//*****************************************************************************
//* RockRun
//*----------------------------------------------------------------------------
//* Version 2 July 2000
//* Written by Kev Stainwright.
//* This game and it's source is freeware.
//*----------------------------------------------------------------------------
//*
//*****************************************************************************

#include <pilot.h>
#include <scrdriver.h>
#include <keymgr.h>

#define sprintf StrPrintF

#include "rockrun.h"

#define APPID  'RKRN'
#define PREFID (APPID+1) //appid + version

//*****************************************************************************
#define FPS 5

#define scnw   160
#define scnh   160
#define scnbpr (scnw/8)
#define scnlpr (scnw/32)

struct
{
    BitmapType bt; //the pixel data is assumed to follow this struct
    char pixels[scnh][scnw/8];
    long failsafe[scnlpr]; //safe guard for bugs
} scrn={{/*w*/scnw,/*h*/scnh,/*bpr*/scnw/8,/*flags*/{0},/*bpp*/1,/*ver*/1,0,0,0},0};

//*****************************************************************************
#define hiscore_count 6

#define cave_maxwidth  40
#define cave_maxheight 22

struct //warning: this struct MUST NOT contain any pointers!
{
    long id;

    char map_char[cave_maxheight][cave_maxwidth];
    char map_flag[cave_maxheight][cave_maxwidth];

    long reqcaveload; //cave load request
    long reqloselife; //lose life request

    long st_activemenu; //in a menu?
    long st_paused; //game paused
    long st_dying; //player dying, but not necessaraly game over
    long st_starting; //starting/restarting cave
    long st_finishing; //cave has been completed

    long flashpaths;
    long slimegrowtime;
    long enabletimer;
    long framecounter;
    long startcave;
    long currentcave;
    long score;
    long recent_hiscore;
    long keys;
    long kflush;
    long playercid; //player movement direction graphic
    long soundid,soundstage;

    //enter name stuff
    char name[4];
    long namei;

    long timeleft,subtime;
    long animcounter;
    long diamondcounter;
    long lives;
    long playerx;
    long playery;
    long rockdelay; //helps to give rocks weight when pusing, and a 'stuck' factor when falling
    long exitisopen;
    long lastcompletedcave;

    //menu stuff
    long ehnx,ehny; //enter hiscore name
    long menusel;

    struct
    {
        long score;
        char startcave,endcave;
        char name[4];
    } hiscores[hiscore_count];
} gs;

long quit,soundvol;

//*****************************************************************************
#define scnmapw (scnw/8) //chars accross screen
#define scnmaph (scnh/8) //chars down screen

#define stats_height 1 //status panel height

#define kUP    keyBitPageUp
#define kDOWN  keyBitPageDown
#define kLEFT  keyBitHard1 //date-key
#define kRIGHT keyBitHard2 //address-key
#define kSTAND keyBitHard3 //todo-key
#define kMENU  keyBitHard4 //memo-key

//*****************************************************************************
//* Menu vars
//*****************************************************************************
enum{
    mid_none=0,
    mid_main,
    mid_hiscores,
    mid_about,
    mid_congrats,
    mid_entername,
};

//*****************************************************************************
//* This sound stuff is a bit naff, the sounds are kinda limited!
//*****************************************************************************
enum //make sure this matches 'sndtab'
{
    sid_well_done,
    sid_menu_open,
    sid_game_over,
    sid_exit_open,
    sid_low_time,
    sid_collect_diamond,
    sid_rock_to_diamond,
    sid_explosion,
    sid_diamond_land,
    sid_push_rock,
    sid_slimegrow,
    sid_rock_land,
    sid_start_playing,
    sid_done_cave,
    sid_none,
    sid_count
};

struct
{
    short freq1,freq2,dur,stages,rand;
}
sndtab[sid_count]= //the order IS the priority, first=most important
{
    {4000,6000,90,4,1}, //sid_well_done,
    { 800,1000,90,2,0}, //sid_menu_open,
    { 100,  50,90,6,0}, //sid_game_over,
    {1000,2000,60,3,0}, //sid_exit_open,
    { 800,   0,30,1,0}, //sid_low_time,
    {3000,4000,30,1,1}, //sid_collect_diamond,
    {1800,2200,40,1,1}, //sid_rock_to_diamond,
    {  50, 100,90,3,1}, //sid_explosion,
    {2500,3000,30,1,1}, //sid_diamond_land,
    { 150, 200,90,1,1}, //sid_push_rock,
    { 200, 300,30,1,1}, //sid_slimegrow,
    { 100, 150,30,1,1}, //sid_rock_land,
    { 300, 700,30,2,0}, //sid_start_playing,
    {3000,1000,30,6,0}, //sid_done_cave,
    {   0,   0, 0,0,0}, //sid_none
};

//*****************************************************************************
void memset(void *p, char v, long s)
{
    for(char *c=(char*)p; s>0; s--) *c++=v;
}

//*****************************************************************************
void PrintHex(long x, long y, unsigned long num)
{
    char buf[]="12345678";
    char h2a[]="0123456789ABCDEF";
    for(long j=7; j>=0; j--,num>>=4) buf[j]=h2a[num&0x0f];
    WinDrawChars(buf,8,x,y);
}

//*****************************************************************************
void ProcessSounds()
{
    if(gs.soundid==sid_none) return; //nothing to do?

    long f;
    if(sndtab[gs.soundid].rand) //rand sound range?
    {
        f=sndtab[gs.soundid].freq2-sndtab[gs.soundid].freq1;
        if(f>0) f=SysRandom(0)%f;
        f+=sndtab[gs.soundid].freq1;
    }
    else //interpolate freq
    {
        long f1=sndtab[gs.soundid].freq1;
        long f2=sndtab[gs.soundid].freq2;
        long ss=sndtab[gs.soundid].stages;
        f=f1+( ((f2-f1)*gs.soundstage) /ss);
    }
    SndCommandType sndCmd;
    sndCmd.cmd   =sndCmdFrqOn;
    sndCmd.param1=f;
    sndCmd.param2=sndtab[gs.soundid].dur;
    sndCmd.param3=soundvol;
    SndDoCmd(0,&sndCmd,true);

    gs.soundstage++;
    if(gs.soundstage==sndtab[gs.soundid].stages) gs.soundid=sid_none; //clear old sound
}

//*****************************************************************************
void ReqSound(long sid)
{
    if(sid>gs.soundid) return; //new req not as important as current sound?
    gs.soundid=sid;
    gs.soundstage=0;
}

//*****************************************************************************
void SetMenu(long mid)
{
    gs.st_activemenu=mid;

    //reset all menu vars
    gs.menusel=0;
    gs.ehnx=2;
    gs.ehny=2;
    memset(gs.name,'.',sizeof(gs.name));
    gs.namei=0;
}

//*****************************************************************************
void ProcessExit()
{
    gs.exitisopen=(gs.diamondcounter<1);

    if(gs.exitisopen)
    {
        if(gs.flashpaths>1) gs.flashpaths--;
        if(gs.flashpaths==0)
        {
            ReqSound(sid_exit_open);
            gs.flashpaths=6;
        }
    }

    //turn exit flash animation on/off
    char_info[EX].animframes=gs.exitisopen ? 2:1;

    //turn empty-space flash animation on/off
    char_info[ES].bitmap[0]=((gs.flashpaths>1) && (gs.flashpaths&2)) ? char_ES2:char_ES1;
}

//*****************************************************************************
void ReqLoadCave(long cave)
{
    if(cave<0) cave=0;
    if(cave>=cave_count) cave=cave_count-1;
    gs.reqcaveload=cave;
}

//*****************************************************************************
void KeyFlushQuick()
{
    gs.kflush=1;
    gs.keys=0;
}

void KeyFlushSlow()
{
    gs.kflush=5;
    gs.keys=0;
}

//*****************************************************************************
void ProcessReqLoadCave()
{
char *tcave_map;
long x,y;

    if(gs.reqcaveload==-1) return;

    KeyFlushSlow();

    //make sure the cave number is valid
    if(gs.reqcaveload<0) gs.reqcaveload=0;
    if(gs.reqcaveload>=cave_count) gs.reqcaveload=cave_count-1;

    //go get its stats
    tcave_map        =cave_maps    [gs.reqcaveload];
    gs.diamondcounter=cave_diamonds[gs.reqcaveload];
    gs.timeleft      =cave_time    [gs.reqcaveload];
    gs.subtime       =FPS;

    //load cave map
    memset(gs.map_char,MP,sizeof(gs.map_char));
    memset(gs.map_flag,0,sizeof(gs.map_flag)); //clear all charmap flags
    for(y=0; y<cave_height; y++)
    {
        for(x=0; x<cave_width; x+=2,tcave_map++) //note: map uses nybbles!
        {
            gs.map_char[y][x+0]=((*tcave_map)>>4)&0x0f;
            gs.map_char[y][x+1]=(*tcave_map)&0x0f;
        }
    }

    //find start pos
    for(y=0; y<cave_height; y++) for(x=0; x<cave_width; x++)
    {
        if(gs.map_char[y][x]==SP) {gs.playerx=x; gs.playery=y;}
    }

    gs.currentcave=gs.reqcaveload;
    gs.reqcaveload=-1;

    //reset new cave vars
    gs.st_starting=6;
    gs.st_paused=0;
    gs.st_dying=0;
    gs.st_finishing=0;
    gs.slimegrowtime=0;
    gs.flashpaths=0;
    gs.soundid=sid_none;
}

//*****************************************************************************
void ExplodeArea(long ex, long ey, long diamonds)
{
long x,y;
char f=diamonds ? cdf_makediamond:0;

    ReqSound(sid_explosion);

    for(y=-1; y<2; y++) for(x=-1; x<2; x++)
    {
        if(char_info[gs.map_char[ey+y][ex+x]].flags&cf_nodestory) continue;
        gs.map_char[ey+y][ex+x]=EP1;
        gs.map_flag[ey+y][ex+x]=f;
    }
}

//*****************************************************************************
void ProcessExplosions()
{
long x,y;

    if(gs.st_paused) return;

    for(y=cave_height-2; y>=0; y--)
    {
        for(x=0; x<cave_width; x++)
        {
            if(!(char_info[gs.map_char[y][x]].flags&cf_exploding)) continue; //not exploding?

            gs.map_char[y][x]++;
            if(gs.map_char[y][x]>EP4) //done exploding?
            {
                gs.map_char[y][x]=(gs.map_flag[y][x]&cdf_makediamond) ? DI:ES;
            }
        }
    }
}

//*****************************************************************************
long IsHiscore(long score)
{
    for(long j=0; j<hiscore_count; j++)
    {
        if(score>gs.hiscores[j].score) return(j+1);
    }
    return(0);
}

//*****************************************************************************
void InsertScore(long score, char startcave, char endcave, char *name)
{
    long j=IsHiscore(score)-1;
    if(j==-1) return;

    for(long k=hiscore_count-1; k>j; k--) gs.hiscores[k]=gs.hiscores[k-1];
    gs.hiscores[j].score=score;
    gs.hiscores[j].name[0]=name[0];
    gs.hiscores[j].name[1]=name[1];
    gs.hiscores[j].name[2]=name[2];
    gs.hiscores[j].name[3]=name[3];
    gs.hiscores[j].startcave=startcave;
    gs.hiscores[j].endcave=(endcave<cave_count) ? endcave:(cave_count-1);
    gs.recent_hiscore=j;
}

//*****************************************************************************
void ResetDefaults()
{
    quit=0;
    memset(&gs,0,sizeof(gs));

    for(long j=0; j<hiscore_count; j++) //reset hiscore table
    {
        InsertScore((j+1)*10,0,0,"KEVS");
    }
    gs.recent_hiscore=-1;

    SetMenu(mid_main);
    gs.enabletimer=true;
    gs.reqcaveload=-1;
    gs.lastcompletedcave=-1;
    gs.soundid=sid_none;
}

//*****************************************************************************
void AppInit()
{
    //look this is weird, I'll confess that I dont know why, but if this code
    //isn't used then the A-B-C and 1-2-3 controls crash the Palm!!??
    FrmSetActiveForm(FrmNewForm(1000,0,0,0,scnw,scnh,0,0,0,0)); //create and activate a fullscreen form

    //set screen mode
    DWord w=scnw,h=scnh,b=1;
    ScrDisplayMode(scrDisplayModeSet,&w,&h,&b,0);

    SysRandom(TimGetTicks()|1);

    //load saved game state
    Word size=sizeof(gs);
    gs.id=0;
    PrefGetAppPreferences(APPID,0,&gs,&size,true);
    if(gs.id!=PREFID) //is it ok?
    {
        ResetDefaults();
    }

    soundvol=PrefGetPreference(prefGameSoundVolume);
}

//*****************************************************************************
void LoseLifeRequest()
{
    gs.reqloselife=true;
    if(gs.lives==0) ReqSound(sid_game_over); //game over?
}

//*****************************************************************************
void ProcessDying()
{
    if(gs.reqloselife)
    {
        gs.reqloselife=false;
        ExplodeArea(gs.playerx,gs.playery,false);
        gs.st_dying=1;
        KeyFlushSlow();
    }

    if(!gs.st_dying) return;

    if(gs.keys&(kUP|kDOWN|kLEFT|kRIGHT|kSTAND|kMENU))
    {
        gs.st_dying=0;
        if(gs.lives>0) //not lost all lives yet?
        {
            gs.lives--;
            ReqLoadCave(gs.currentcave); //restart
        }
        else //game over
        {
            if(IsHiscore(gs.score)) //got a hiscore?
            {
                ReqSound(sid_well_done);
                SetMenu(mid_entername);
            }
            else
            {
                SetMenu(mid_main);
            }
        }
    }
}

//*****************************************************************************
void AppKill()
{
    while(EvtEventAvail())
    {
        EventType event;
        EvtGetEvent(&event,evtWaitForever);
    }

    //save game state
    gs.id=PREFID;
    PrefSetAppPreferences(APPID,0,1,&gs,sizeof(gs),true);
}

//*****************************************************************************
void DrawChar(long x, long y, char c)
{
    if(c>=CHAR_COUNT) c=' ';
    char *bitmap=char_info[c].bitmap[gs.animcounter%char_info[c].animframes];
    y<<=3;
    scrn.pixels[y++][x]=*bitmap++;
    scrn.pixels[y++][x]=*bitmap++;
    scrn.pixels[y++][x]=*bitmap++;
    scrn.pixels[y++][x]=*bitmap++;
    scrn.pixels[y++][x]=*bitmap++;
    scrn.pixels[y++][x]=*bitmap++;
    scrn.pixels[y++][x]=*bitmap++;
    scrn.pixels[y  ][x]=*bitmap  ;
}

//*****************************************************************************
long DrawString(long x, long y, char *str)
{
    long j;
    for(j=0; str[j]; j++) DrawChar(x+j,y,str[j]);
    return(j);
}

//*****************************************************************************
void DrawMap()
{
long x,y;
long xx=0,yy=0;

    xx=gs.playerx-(scnmapw/2);
    yy=gs.playery-(scnmaph/2);

    if(xx<0) xx=0;
    else if((xx+scnmapw)>cave_width) xx=cave_width-scnmapw;
    if(yy<0) yy=0;
    else if((yy+(scnmaph-stats_height))>cave_height) yy=cave_height-(scnmaph-stats_height);

    for(y=0; y<scnmaph-stats_height; y++)
    {
        for(x=0; x<scnmapw; x++)
        {
            DrawChar(x,y+stats_height,gs.map_char[yy+y][xx+x]);
        }
    }
}

//*****************************************************************************
void ProcessFinishing()
{
    if(!gs.st_finishing) return;

    //if we've got time left, add that to our score
    if(gs.enabletimer && gs.timeleft>0)
    {
        gs.score+=6;
        gs.timeleft-=6;
        if(gs.timeleft<0) gs.timeleft=0;
    }
    else
    {
        gs.st_finishing--; //dramatic pause
        if(!gs.st_finishing)
        {
            if(gs.currentcave>gs.lastcompletedcave) gs.lastcompletedcave=gs.currentcave;
            gs.currentcave++;
            if(gs.currentcave>=cave_count)
            {
                ReqSound(sid_well_done);
                SetMenu(mid_congrats);
            }
            ReqLoadCave(gs.currentcave); //next cave
        }
    }
}

//*****************************************************************************
void ProcessGameKeys(long *rpx, long *rpy, long *stand)
{
    *rpx=gs.playerx;
    *rpy=gs.playery;
    *stand=false;
    gs.playercid=PI; //idle

    if(gs.keys&kMENU)
    {
        ReqSound(sid_menu_open);
        gs.st_paused=true;
        KeyFlushSlow();
    }

    if(gs.keys&kSTAND) *stand=true;

         if(gs.keys&kUP   ) {*rpy-=1; gs.playercid=PU;}
    else if(gs.keys&kDOWN ) {*rpy+=1; gs.playercid=PD;}
    else if(gs.keys&kLEFT ) {*rpx-=1; gs.playercid=PL;}
    else if(gs.keys&kRIGHT) {*rpx+=1; gs.playercid=PR;}
}

//*****************************************************************************
void ProcessDeadlyThingsNearPlayer()
{
    if(gs.st_dying) return; //busy dying?
    if(gs.st_starting) return; //busy starting?
    if(gs.st_finishing) return; //busy finishing?
    if(gs.st_paused) return; //busy paused?

    //does the thing above us fall?
    if(char_info[gs.map_char[gs.playery-1][gs.playerx]].flags&cf_falls)
    {
        //is the thing above us falling on us?
        if(gs.map_flag[gs.playery-1][gs.playerx]&cdf_isfalling)
        {
            LoseLifeRequest();
            return;
        }
    }

    //are we next to anything deadly?
    if((char_info[gs.map_char[gs.playery+0][gs.playerx-1]].flags&cf_death) || //left
       (char_info[gs.map_char[gs.playery+0][gs.playerx+1]].flags&cf_death) || //right
       (char_info[gs.map_char[gs.playery-1][gs.playerx+0]].flags&cf_death) || //up
       (char_info[gs.map_char[gs.playery+1][gs.playerx+0]].flags&cf_death))   //down
    {
        LoseLifeRequest();
    }
}

//*****************************************************************************
void ProcessPlayerMoves()//long rpx, long rpy)
{
    if(gs.st_dying) return; //busy dying?
    if(gs.st_starting) return; //busy starting?
    if(gs.st_finishing) return; //busy finishing?
    if(gs.st_paused) return; //busy paused?

    long rpx=1,rpy=1,stand;
    ProcessGameKeys(&rpx,&rpy,&stand);

    gs.map_char[gs.playery][gs.playerx]=ES; //clear player from old pos

    if(char_info[gs.map_char[rpy][rpx]].flags&cf_diamond) //got a diamond?
    {
        if(gs.diamondcounter>0) gs.diamondcounter--;
        ReqSound(sid_collect_diamond);
        gs.score+=15;
    }

    if(stand) //standing and grabing?
    {
        if(char_info[gs.map_char[rpy][rpx]].flags&cf_standgrab) //can we grab/clear this?
        {
            gs.map_char[rpy][rpx]=ES;
        }
    }
    else //trying to move?
    {
        if(char_info[gs.map_char[rpy][rpx]].flags&cf_exit) //found the exit door?
        {
            if(gs.exitisopen)
            {
                ReqSound(sid_done_cave);
                gs.st_finishing=10;
                gs.playerx=rpx;
                gs.playery=rpy;
            }
        }
        else if(char_info[gs.map_char[rpy][rpx]].flags&cf_death) //does this kill us?
        {
            LoseLifeRequest();
        }
        else if(char_info[gs.map_char[rpy][rpx]].flags&cf_push) //can we push this?
        {
            //calc the rock's pushed pos
            long ppx=rpx+(rpx-gs.playerx);
            if(gs.rockdelay==0 && gs.map_char[gs.playery][ppx]==ES) //can rock be pushed?
            {
                ReqSound(sid_push_rock);
                gs.map_char[gs.playery][ppx]=gs.map_char[gs.playery][rpx]; //move rock into pushed pos
                gs.playerx=rpx;
            }
        }
        else if(!(char_info[gs.map_char[rpy][rpx]].flags&cf_nomove)) //can we move into the requested pos?
        {
            gs.playerx=rpx;
            gs.playery=rpy;
        }
    }

    gs.map_char[gs.playery][gs.playerx]=gs.playercid; //plot player in new pos
}

//*****************************************************************************
void BlitScreen()
{
    WinDrawBitmap((BitmapPtr)&scrn,0,0);
}

//*****************************************************************************
void ProcessDelays()
{
    if(gs.rockdelay<1) gs.rockdelay=4;
    else gs.rockdelay--;
}

//*****************************************************************************
void ProcessTimer()
{
    if(gs.st_paused) return;
    if(gs.st_finishing) return;
    if(gs.st_dying) return;
    if(gs.st_starting) return;

    if(gs.enabletimer)
    {
        if(--gs.subtime<0)
        {
            gs.subtime=FPS;
            gs.timeleft--;
            if(gs.timeleft<10) ReqSound(sid_low_time);
            if(gs.timeleft<0) LoseLifeRequest(); //out of time
        }
    }

    if(gs.timeleft<  0) gs.timeleft=  0;
    if(gs.timeleft>999) gs.timeleft=999;
}

//*****************************************************************************
void DrawStats()
{
char buf[32];

    if(gs.diamondcounter<  0) gs.diamondcounter=  0;
    if(gs.diamondcounter>999) gs.diamondcounter=999;

    if(gs.lives< 0) gs.lives= 0;
    if(gs.lives>99) gs.lives=99;

    if(gs.timeleft<  0) gs.timeleft=  0;
    if(gs.timeleft>999) gs.timeleft=999;

    if(gs.score<    0) gs.score=    0;
    if(gs.score>99999) gs.score=99999;

    if(gs.currentcave<          0) gs.currentcave=0;
    if(gs.currentcave>=cave_count) gs.currentcave=cave_count-1;

    if(gs.st_starting>0)
    {
        sprintf(buf,"       CAVE %c       ",(char)('A'+gs.currentcave));
        DrawString(0,0,buf);
        return;
    }

    //stats "D000 L00 T000 S00000"
    sprintf(buf,"\x07%03ld \x11%02ld T%03ld S%05ld",gs.diamondcounter,gs.lives,gs.timeleft,gs.score);
    DrawString(0,0,buf);
}

//*****************************************************************************
void ProcessFallingStuff()
{
long x,y,fx,fy,wasfalling;

    if(gs.st_paused) return;

    for(y=cave_height-2; y>=0; y--)
    {
        for(x=0; x<cave_width; x++)
        {
            if(!(char_info[gs.map_char[y][x]].flags&cf_falls)) continue; //not a fallable object?

            wasfalling=gs.map_flag[y][x]&cdf_isfalling;
            gs.map_flag[y][x]&=~cdf_isfalling;
            if(wasfalling && gs.map_char[y][x]==RK && gs.map_char[y+1][x]==MB) //rock falling on magic brick?
            {
                ReqSound(sid_rock_to_diamond);
                if(gs.map_char[y+2][x]==ES) //can rock fall through magic brick?
                {
                    gs.map_char[y][x]=DI; //change rock to diamond
                    fx=x;
                    fy=y+2;
                }
                else //rock vanishes
                {
                    gs.map_char[y][x]=ES;
                    continue;
                }
            }
            else if(gs.map_char[y+1][x]==ES) //can fall straight down?
            {
                //continue to fall if already started; else only start falling
                //when the delay is right, helps to give a sense of 'stuckness'
                if(wasfalling || (gs.rockdelay&1)) {fx=x; fy=y+1;}
                else continue;
            }
            else if(char_info[gs.map_char[y+1][x]].flags&cf_nosupport) //sitting on none-supporting ground?
            {
                if(gs.map_char[y][x-1]==ES && gs.map_char[y+1][x-1]==ES) {fx=x-1; fy=y;} //fall left?
                else if(gs.map_char[y][x+1]==ES && gs.map_char[y+1][x+1]==ES) {fx=x+1; fy=y;} //fall right?
                else //cant fall
                {
                    if(wasfalling) ReqSound((gs.map_char[y][x]==DI) ? sid_diamond_land:sid_rock_land);
                    continue;
                }
            }
            else //cant fall
            {
                if(wasfalling) //are we falling on monster?
                {
                    ReqSound((gs.map_char[y][x]==DI) ? sid_diamond_land:sid_rock_land);
                    if(gs.map_char[y+1][x]==BF) ExplodeArea(x,y,true);
                    else if(gs.map_char[y+1][x]==BM) ExplodeArea(x,y,false);
                }
                continue;
            }
            gs.map_char[fy][fx]=gs.map_char[y][x];
            gs.map_flag[fy][fx]|=cdf_isfalling;
            gs.map_char[y][x]=ES;
        }
    }
}

//*****************************************************************************
void ProcessBoxMonster(long x, long y) //left-handed
{
    //handle exploding slime
    if((char_info[gs.map_char[y+0][x-1]].flags&cf_slime) ||
       (char_info[gs.map_char[y+0][x+1]].flags&cf_slime) ||
       (char_info[gs.map_char[y-1][x+0]].flags&cf_slime) ||
       (char_info[gs.map_char[y+1][x+0]].flags&cf_slime))
    {
        ExplodeArea(x,y,true); //creates diamonds (orig game does not!)
        return;
    }

    long mfx,mfy; //move forward
    long llx,lly; //look left
    long tl,tr;   //turn left/right
    char f=gs.map_flag[y][x]|cdf_aiupdated;

           if((f&cdf_movemask)==cdf_moveup   )   {mfx= 0; mfy=-1; llx=-1; lly= 0; tl=cdf_moveleft ; tr=cdf_moveright;}
    else   if((f&cdf_movemask)==cdf_moveleft )   {mfx=-1; mfy= 0; llx= 0; lly=+1; tl=cdf_movedown ; tr=cdf_moveup   ;}
    else   if((f&cdf_movemask)==cdf_movedown )   {mfx= 0; mfy=+1; llx=+1; lly= 0; tl=cdf_moveright; tr=cdf_moveleft ;}
    else /*if((f&cdf_movemask)==cdf_moveright)*/ {mfx=+1; mfy= 0; llx =0; lly=-1; tl=cdf_moveup   ; tr=cdf_movedown ;}

    //if !previously_turned_left && left_is_ES then turn_left and return
    if(!(f&cdf_turnedtoES) && gs.map_char[y+lly][x+llx]==ES)
    {
        f&=~cdf_movemask;
        gs.map_flag[y][x]=f|tl|cdf_turnedtoES;
        return;
    }
    f&=~cdf_turnedtoES;

    if(gs.map_char[y+mfy][x+mfx]==ES) //can we move forward?
    {
        gs.map_char[y+mfy][x+mfx]=gs.map_char[y][x];
        gs.map_flag[y+mfy][x+mfx]=f;
        gs.map_char[y][x]=ES;
        gs.map_flag[y][x]=0;
    }
    else //dead end, so turn right
    {
        f&=~cdf_movemask;
        f|=tr;
        gs.map_flag[y][x]=f;
    }
}

//*****************************************************************************
void ProcessButterflyMonster(long x, long y) //right-handed
{
    //handle exploding slime
    if((char_info[gs.map_char[y+0][x-1]].flags&cf_slime) ||
       (char_info[gs.map_char[y+0][x+1]].flags&cf_slime) ||
       (char_info[gs.map_char[y-1][x+0]].flags&cf_slime) ||
       (char_info[gs.map_char[y+1][x+0]].flags&cf_slime))
    {
        ExplodeArea(x,y,true); //creates diamonds
        return;
    }

    long mfx,mfy; //move forward
    long lrx,lry; //look right
    long tl,tr;   //turn left/right
    char f=gs.map_flag[y][x]|cdf_aiupdated;

           if((f&cdf_movemask)==cdf_moveup   )   {mfx= 0; mfy=-1; lrx=+1; lry= 0; tl=cdf_moveleft ; tr=cdf_moveright;}
    else   if((f&cdf_movemask)==cdf_moveleft )   {mfx=-1; mfy= 0; lrx= 0; lry=-1; tl=cdf_movedown ; tr=cdf_moveup   ;}
    else   if((f&cdf_movemask)==cdf_movedown )   {mfx= 0; mfy=+1; lrx=-1; lry= 0; tl=cdf_moveright; tr=cdf_moveleft ;}
    else /*if((f&cdf_movemask)==cdf_moveright)*/ {mfx=+1; mfy= 0; lrx =0; lry=+1; tl=cdf_moveup   ; tr=cdf_movedown ;}

    //if !previously_turned_right && right_is_ES then turn_right and return
    if(!(f&cdf_turnedtoES) && gs.map_char[y+lry][x+lrx]==ES)
    {
        f&=~cdf_movemask;
        gs.map_flag[y][x]=f|tr|cdf_turnedtoES;
        return;
    }
    f&=~cdf_turnedtoES;

    if(gs.map_char[y+mfy][x+mfx]==ES) //can we move forward?
    {
        gs.map_char[y+mfy][x+mfx]=gs.map_char[y][x];
        gs.map_flag[y+mfy][x+mfx]=f;
        gs.map_char[y][x]=ES;
        gs.map_flag[y][x]=0;
    }
    else //dead end, so turn left
    {
        f&=~cdf_movemask;
        f|=tl;
        gs.map_flag[y][x]=f;
    }
}

//*****************************************************************************
#define slimelistmax 100
long slimelistcount;

struct {char x,y;} slimelist[slimelistmax];

//*****************************************************************************
void ResetSlimeList()
{
    slimelistcount=0;
    if(gs.slimegrowtime==0) //starting to wait for a new piece of slime?
    {
        gs.slimegrowtime=gs.framecounter+3+(SysRandom(0)%15);
    }
    else if(gs.slimegrowtime<gs.framecounter) //ready to grow yet?
    {
        gs.slimegrowtime=0; //signal grow-now
    }
}

//*****************************************************************************
void AddSlimeToList(long x, long y)
{
    if(gs.slimegrowtime) return; //waiting to grow?
    if(slimelistcount>=slimelistmax) return; //too many already?

    if(char_info[gs.map_char[y-1][x+0]].flags&cf_slimegrow ||
       char_info[gs.map_char[y+1][x+0]].flags&cf_slimegrow ||
       char_info[gs.map_char[y+0][x-1]].flags&cf_slimegrow ||
       char_info[gs.map_char[y+0][x+1]].flags&cf_slimegrow)
    {
        slimelist[slimelistcount].x=x;
        slimelist[slimelistcount].y=y;
        slimelistcount++;
    }
}

//*****************************************************************************
void ProcessSlimeGrowth()
{
    if(slimelistcount<1) return;

    ReqSound(sid_slimegrow);

    char d[4][2]={{-1,0},{+1,0},{0,-1},{0,+1}};
    long si=SysRandom(0)%slimelistcount;
    long di=SysRandom(0)%4;
    long x=slimelist[si].x;
    long y=slimelist[si].y;
    for(long j=0; j<3; j++)
    {
        long dx=d[di][0];
        long dy=d[di][1];
        if(char_info[gs.map_char[y+dy][x+dx]].flags&cf_slimegrow)
        {
            gs.map_char[y+dy][x+dx]=GS;
            break;
        }
        di=(di+1)%4;
    }
}

//*****************************************************************************
void ProcessAI()
{
long x,y;

    if(gs.st_paused) return;

    ResetSlimeList();
    for(y=cave_height-2; y>=0; y--)
    {
        for(x=0; x<cave_width; x++)
        {
            if(!(char_info[gs.map_char[y][x]].flags&cf_ai)) continue; //not AI thingy?

            if(gs.map_flag[y][x]&cdf_aiupdated) //already updated?
            {
                gs.map_flag[y][x]&=~cdf_aiupdated; //clear updated flag
                continue;
            }

            switch(gs.map_char[y][x])
            {
                case BM: ProcessBoxMonster(x,y); break;
                case BF: ProcessButterflyMonster(x,y); break;
                case GS: AddSlimeToList(x,y); break;
            }
        }
    }
    ProcessSlimeGrowth();
}

//*****************************************************************************
void ProcessFrameRate(long stage)
{
static long tm=0;

    if(stage) //end of frame?
    {
        //sleep for any spare frame time        
        tm=(SysTicksPerSecond()/FPS)-(TimGetTicks()-tm);
        if(tm>0) SysTaskDelay(tm);
    }
    else //start of frame?
    {
        tm=TimGetTicks();
    }
}

//*****************************************************************************
void ProcessPalmEvents()
{
    if(gs.kflush>0) {gs.kflush--; gs.keys=0;}
    else gs.keys=KeyCurrentState()&(kUP|kDOWN|kLEFT|kRIGHT|kSTAND|kMENU);

    for(long c=0; c<10 && EvtEventAvail(); c++)
    {
        EventType event;
        EvtGetEvent(&event,0);

        if(event.eType==appStopEvent) quit=1;

        if((event.eType==keyDownEvent) &&
           (event.data.keyDown.modifiers&commandKeyMask) &&
           (event.data.keyDown.chr==pageUpChr ||
            event.data.keyDown.chr==pageDownChr ||
            event.data.keyDown.chr==hard1Chr ||
            event.data.keyDown.chr==hard2Chr ||
            event.data.keyDown.chr==hard3Chr ||
            event.data.keyDown.chr==hard4Chr)
          )
        {
            //block up/dpwn/app hard keys
        }
        else SysHandleEvent(&event);
    }
}

//*****************************************************************************
void ProcessStarting()
{
    if(!gs.st_starting) return;
    gs.st_starting--;
    if(!gs.st_starting) ReqSound(sid_start_playing);
}

//*****************************************************************************
void ProcessPause()
{
    if(!gs.st_paused) return;

    static long sel=2;
    long x=5,y=7;
    DrawString(x,y+0,"          ");
    DrawString(x,y+1," PAUSED   ");
    DrawString(x,y+2," CONTINUE ");
    DrawString(x,y+3," RESTART  ");
    DrawString(x,y+4," QUIT     ");
    DrawString(x,y+5,"          ");
    DrawChar(x,y+sel,CR);

    if(gs.keys&kUP   && sel>2) sel--;
    if(gs.keys&kDOWN && sel<4) sel++;

    switch(sel)
    {
    case 2: //continue
        if(gs.keys&(kMENU|kSTAND)) {gs.st_paused=false; sel=2;}
        break;
    case 3: //restart
        if(gs.keys&(kMENU|kSTAND)) {gs.st_paused=false; sel=2; LoseLifeRequest();}
        break;
    case 4: //quit
        if(gs.keys&(kMENU|kSTAND)) {SetMenu(mid_main); sel=2;}
        break;
    }

    if(gs.keys) KeyFlushQuick();
}

//*****************************************************************************
void ProcessMainMenu()
{
    if(gs.keys&kUP   && gs.menusel>0) gs.menusel--;
    if(gs.keys&kDOWN && gs.menusel<4) gs.menusel++;

    switch(gs.menusel)
    {
    case 0: //cave select
        if(gs.keys&kLEFT  && gs.startcave>0) gs.startcave--;
        if(gs.keys&kRIGHT && gs.startcave<=gs.lastcompletedcave && gs.startcave<(cave_count-1)) gs.startcave++;
        if(gs.keys&(kMENU|kSTAND))
        {
            SetMenu(mid_none);
            gs.score=0;
            gs.lives=3;
            ReqLoadCave(gs.startcave);
        }
        break;
    case 1: //timer
        if(gs.keys&(kMENU|kSTAND|kLEFT|kRIGHT)) gs.enabletimer=!gs.enabletimer;
        break;
    case 2: //hiscores
        if(gs.keys&(kMENU|kSTAND|kLEFT|kRIGHT)) SetMenu(mid_hiscores);
        break;
    case 3: //about
        if(gs.keys&(kMENU|kSTAND|kLEFT|kRIGHT)) SetMenu(mid_about);
        break;
    case 4: //quit
        if(gs.keys&(kMENU|kSTAND|kLEFT|kRIGHT)) quit=true;
        break;
    }

    if(gs.keys) KeyFlushQuick();
}

//*****************************************************************************
void ProcessSimpleMenu()
{
    if(gs.keys)
    {
        SetMenu(mid_main);
        KeyFlushQuick();
    }
}

//*****************************************************************************
void ProcessAboutMenu()
{
    if(gs.keys&(kLEFT|kRIGHT|kSTAND|kMENU))
    {
        SetMenu(mid_main);
        KeyFlushQuick();
        return;
    }

    if(gs.keys&kUP   && gs.menusel>0) {gs.menusel--; KeyFlushQuick();}
    if(gs.keys&kDOWN && gs.menusel<1) {gs.menusel++; KeyFlushQuick();}

    static long cheat=0;
    if(gs.keys&kDOWN)
    {
        if(++cheat>5)
        {
            gs.lastcompletedcave=cave_count-1;
            cheat=0;
            ReqSound(sid_well_done); //you cheat!
        }
    }
    else cheat=0;
}

//*****************************************************************************
void ProcessMenu()
{
    switch(gs.st_activemenu)
    {
        default           : ProcessMainMenu();   break;
        case mid_hiscores : ProcessSimpleMenu(); break;
        case mid_entername: break;
        case mid_about    : ProcessAboutMenu();  break;
        case mid_congrats : break;
    }
}

//*****************************************************************************
void DrawMainMenu()
{
long x,y;

    for(y=0; y<20; y++)
    {
        for(x=0; x<20; x++)
        {
            DrawChar(x,y,menumap_main[y][x]);
        }
    }
    DrawChar(4-1,14+gs.menusel-1,CR);
    DrawChar(15-1,14-1,(char)('A'+gs.startcave));
    DrawString(11-1,15-1,gs.enabletimer ? (char*)"ON":(char*)"OFF");
}

//*****************************************************************************
void DrawHiscoresMenu()
{
    char buf[32];
    long x,y=0,j;

    for(x=0; x<20; x++) DrawChar(x,y,BR); y++;
    DrawString(0,y++,"                    ");
    DrawString(0,y++,"                    ");
    DrawString(0,y++,"    \x07 HISCORES \x07    ");
    DrawString(0,y++,"                    ");
    DrawString(0,y++,"                    ");
    DrawString(0,y++," SCORE  NAME  CAVES ");
    DrawString(0,y++,"                    ");
    for(j=0; j<hiscore_count; j++,y++)
    {
        sprintf(buf," %05ld  %c%c%c%c   %c-%c  ",
            gs.hiscores[j].score,
            gs.hiscores[j].name[0],
            gs.hiscores[j].name[1],
            gs.hiscores[j].name[2],
            gs.hiscores[j].name[3],
            (char)(gs.hiscores[j].startcave+'A'),
            (char)(gs.hiscores[j].endcave+'A'));
        DrawString(0,y,buf);
        if(j==gs.recent_hiscore)
        {
            DrawChar(7,y,BM);
            DrawChar(12,y,BM);
        }
    }
    DrawString(0,y++,"                    ");
    DrawString(0,y++,"                    ");
    DrawString(0,y++,"     PRESS KEY      ");
    DrawString(0,y++,"                    ");
    DrawString(0,y++,"                    ");
    for(x=0; x<20; x++) DrawChar(x,y,BR); y++;
}

//*****************************************************************************
void DrawEnterNameMenu()
{
    long x,y=0;

    //draw screen
    for(x=0; x<20; x++) DrawChar(x,y,BR); y++;
    DrawString(0,y++,"                    ");
    DrawString(0,y++,"     WELL DONE!     ");
    DrawString(0,y++,"                    ");
    DrawString(0,y++,"YOU'VE GOT A HISCORE");
    DrawString(0,y++,"                    ");
    DrawString(0,y++,"                    ");
    DrawString(0,y++," PLEASE ENTER YOUR  ");
    DrawString(0,y++,"     NAME BELOW     ");
    DrawString(0,y++,"                    ");
    DrawString(0,y++," A   B   C   D   E  ");
    DrawString(0,y++," F   G   H   I   J  ");
    DrawString(0,y++," K   L   M   N   O  ");
    DrawString(0,y++," P   Q   R   S   T  ");
    DrawString(0,y++," U   V   W   X   Y  ");
    DrawString(0,y++," Z   \x07   !  DEL END ");
    DrawString(0,y++,"                    ");
    DrawString(0,y++,"     NAME: KEVS     ");
    DrawString(0,y++,"                    ");
    for(x=0; x<20; x++) DrawChar(x,y,BR); y++;

    //move cursor
    if(gs.keys&kLEFT ) gs.ehnx--;
    if(gs.keys&kRIGHT) gs.ehnx++;
    if(gs.keys&kUP   ) gs.ehny--;
    if(gs.keys&kDOWN ) gs.ehny++;
    if(gs.ehnx<0) gs.ehnx=0;
    if(gs.ehnx>4) gs.ehnx=4;
    if(gs.ehny<0) gs.ehny=0;
    if(gs.ehny>5) gs.ehny=5;

    if(gs.keys&(kSTAND|kMENU)) //selecting?
    {
        x=gs.ehnx+(gs.ehny*5);
        if(x<=27) //letter?
        {
            if(x==26) x=DI;
            else if(x==27) x='!';
            else x+='A';
            gs.name[gs.namei]=x;
            if(gs.namei<3) gs.namei++;
            else x=29; //end name
        }
        if(x==28) //delete?
        {
            gs.name[gs.namei]='.';
            if(gs.namei>0) gs.namei--;
            gs.name[gs.namei]='.';
        }
        if(x==29) //end?
        {
            InsertScore(gs.score,gs.startcave,gs.currentcave,gs.name);
            SetMenu(mid_hiscores);
        }
    }

    if(gs.keys) KeyFlushQuick();

    //draw cursor
    x= 0+(gs.ehnx*4);
    y=10+(gs.ehny);
    if(gs.ehny==5 && gs.ehnx>=3) x--;
    DrawChar(x,y,CR);

    //draw name
    x=11; y=17;
    DrawChar(x++,y,gs.name[0]);
    DrawChar(x++,y,gs.name[1]);
    DrawChar(x++,y,gs.name[2]);
    DrawChar(x++,y,gs.name[3]);
}

//*****************************************************************************
void DrawAboutMenu()
{
    long x,y=0;

    for(x=0; x<20; x++) DrawChar(x,y,BR); y++;
    if(gs.menusel==0)
    {
        DrawString(0,y++,"\x07 ABOUT - PAGE 1/2 \x07");
        DrawString(0,y++,"                    ");
        DrawString(0,y++,"   AVOID BADDIES,   ");
        DrawString(0,y++,"   PUSH ROCKS AND   ");
        DrawString(0,y++,"   COLLECT ENOUGH   ");
        DrawString(0,y++,"    DIAMONDS TO     ");
        DrawString(0,y++,"   OPEN THE EXIT.   ");
        DrawString(0,y++,"                    ");
        DrawString(0,y++,"  CODE AND GFX BY   ");
        DrawString(0,y++,"  KEV STAINWRIGHT   ");
        DrawString(0,y++,"                    ");
        DrawString(0,y++,"    THIS GAME IS    ");
        DrawString(0,y++,"  FREEWARE, ENJOY!  ");
        DrawString(0,y++,"                    ");
        DrawString(0,y++,"  VER 02-JULY-2000  ");
        DrawString(0,y++,"                    ");
        DrawString(0,y++,"UP/DOWN CHANGES PAGE");
        DrawString(0,y++,"  OTHER KEYS QUIT   ");
    }
    else
    {
        DrawString(0,y++,"\x07 ABOUT - PAGE 2/2 \x07");
        DrawString(0,y++,"                    ");
        DrawString(0,y++,"                    ");
        DrawString(0,y++,"     GAME KEYS      ");
        DrawString(0,y++,"                    ");
        DrawString(0,y++,"    DATE - LEFT     ");
        DrawString(0,y++," ADDRESS - RIGHT    ");
        DrawString(0,y++," UP/DOWN - UP/DOWN  ");
        DrawString(0,y++,"    TODO - STAND    ");
        DrawString(0,y++,"    MEMO - PAUSE    ");
        DrawString(0,y++,"                    ");
        DrawString(0,y++,"                    ");
        DrawString(0,y++," YOU CAN CHEAT AND  ");
        DrawString(0,y++," OPEN ALL THE CAVES ");
        DrawString(0,y++,"  BY PRESSING AND   ");
        DrawString(0,y++,"HOLDING THE DOWN-KEY");
        DrawString(0,y++,"   NOW FOR A FEW    ");
        DrawString(0,y++,"      SECONDS!      ");
    }
    for(x=0; x<20; x++) DrawChar(x,y,BR); y++;
}

//*****************************************************************************
void DrawCongratsMenu()
{
    char buf[32];
    long x,y=0;

    for(x=0; x<20; x++) DrawChar(x,y,BR); y++;
    DrawString(0,y++,"                    ");
    DrawString(0,y++,"\x07 CONGRATULATIONS! \x07");
    DrawString(0,y++,"                    ");
    DrawString(0,y++,"                    ");
    DrawString(0,y++," YOU HAVE COMPLETED ");
    DrawString(0,y++,"                    ");
    DrawString(0,y++,"   THE LAST CAVE.   ");
    DrawString(0,y++,"                    ");
    DrawString(0,y++,"                    ");
    DrawString(0,y++,"YOUR FINAL SCORE IS ");
    DrawString(0,y++,"                    ");
    sprintf(buf,"       %05ld        ",gs.score); DrawString(0,y++,buf);
    DrawString(0,y++,"                    ");
    DrawString(0,y++,"                    ");
    DrawString(0,y++,"     WELL DONE      ");
    DrawString(0,y++,"                    ");
    DrawString(0,y++,"     PRESS KEY      ");
    DrawString(0,y++,"                    ");
    for(x=0; x<20; x++) DrawChar(x,y,BR); y++;

    if(gs.keys)
    {
        SetMenu(IsHiscore(gs.score) ? mid_entername:mid_main);
        KeyFlushQuick();
    }
}

//*****************************************************************************
void ProcessRocky()
{
    if(gs.st_activemenu!=mid_main) return;

    static long rkx=1,rky=18,rkd=1,rkc=0;
    long rkg;
    switch(rkd)
    {
        case 0: //up
            if(rky>0 && menumap_main[rky-1][rkx]==ES) rky--;
            else rkc=0;
            rkg=PU;
            break;
        case 1: //right
            if(rkx<19 && menumap_main[rky][rkx+1]==ES) rkx++;
            else rkc=0;
            rkg=PR;
            break;
        case 2: //down
            if(rky<19 && menumap_main[rky+1][rkx]==ES) rky++;
            else rkc=0;
            rkg=PD;
            break;
        default: //left
            if(rkx>0 && menumap_main[rky][rkx-1]==ES) rkx--;
            else rkc=0;
            rkg=PL;
            break;
    }
    DrawChar(rkx,rky,rkg);

    if(--rkc<0) {rkd=SysRandom(0)%4; rkc=SysRandom(0)%10;}
}

//*****************************************************************************
void DrawMenu()
{
    switch(gs.st_activemenu)
    {
        default           : DrawMainMenu();      break;
        case mid_hiscores : DrawHiscoresMenu();  break;
        case mid_entername: DrawEnterNameMenu(); break;
        case mid_about    : DrawAboutMenu();     break;
        case mid_congrats : DrawCongratsMenu();  break;
    }
}

//*****************************************************************************
void DrawMessages()
{
    if((gs.framecounter%10)<5) return;

    if(gs.st_dying)
    {
        if(gs.lives>0) DrawString(0,0,(char*)"     PRESS KEY!     ");
        else           DrawString(0,0,(char*)"  G A M E  O V E R  ");
    }
}

//*****************************************************************************
void AppLoop()
{
    for(; !quit; gs.framecounter++,gs.animcounter++)
    {
        ProcessFrameRate(0);
        ProcessPalmEvents();

        if(gs.st_activemenu==mid_none)
        {
            ProcessReqLoadCave();

            ProcessPlayerMoves();
            ProcessDeadlyThingsNearPlayer();
            ProcessFallingStuff();
            ProcessAI();
            ProcessExplosions();

            ProcessTimer();
            ProcessDelays();
            ProcessExit();

            ProcessStarting();
            ProcessFinishing();
            ProcessDying();

            DrawMap();
            DrawStats();
            DrawMessages();
            ProcessPause();
            BlitScreen();
        }
        else
        {
            ProcessMenu();
            DrawMenu();
            ProcessRocky();
            BlitScreen();
        }

        ProcessSounds();
        ProcessFrameRate(1);
    }
}

//*****************************************************************************
DWord PilotMain(Word LaunchCodeCmd, Ptr LaunchCodeStruct, Word LaunchCodeFlags)
{
    if(LaunchCodeCmd!=sysAppLaunchCmdNormalLaunch) return(0);

    AppInit();
    AppLoop();
    AppKill();
    return(0);
}
