//*****************************************************************************
//* RockRun
//*----------------------------------------------------------------------------
//* Rules for the slime:
//*  - Grows slowly at first.
//*  - Grows only into ES or ER.
//*  - Grows into neighbouring blocks.
//*  - Supports rocks/diamonds.
//*  - Baddies explode on contact (note: no cascades!).
//*  - Turns into rocks when (not sure about this) got enough diamonds.
//*
//* Rules for baddies:
//*  - No cascading explosions! eg: rocks falls on group of boxes.
//*  - When hit with rocks, box monsters DONT create diamonds.
//*  - When hit with rocks, butterfly monsters DO create diamonds.
//*  - Boxes follow left-hand walls, butterflys follow right-hand walls.
//*****************************************************************************

//*****************************************************************************
enum //note, this order MUST match the char_info list!
{
    //only the first 16 can be used to create levels with
    ES=0, //0x00 - (orig=0x00) empty space
    MP,   //0x01 - (orig=0x07) metal plate
    SP,   //0x02 - (orig=0x38) starting point
    EX,   //0x03 - (orig=0x04) exit
    ER,   //0x04 - (orig=0x01) earth (soil)
    RK,   //0x05 - (orig=0x10) rock
    BR,   //0x06 - (orig=0x02) brick
    DI,   //0x07 - (orig=0x14) diamond
    BF,   //0x08 - (orig=0x33,0x30,0x37,0x32) butterfly monster
    GS,   //0x09 - (orig=0x3a) growing slime
    BM,   //0x0a - (orig=0x08,0x0c,0x0a,0x09,0x0b,0x0f) box monster
    MB,   //0x0b - (orig=0x03) magic brick, converts rocks into diamonds
    _12,_13,_14,_15, //unused

    BS,   //0x10 - blocked space
    PI,   //0x11 - player idle
    PL,   //0x12 - player face left
    PR,   //0x13 - player face right
    PU,   //0x14 - player face up
    PD,   //0x15 - player face down
    CR,   //0x16 - cursor
    EP1,  //0x17 - explode1
    EP2,  //0x18 - explode2
    EP3,  //0x19 - explode3
    EP4,  //0x1a - explode4

    _27,_28,_29,_30,_31, //unused

    //ascii chars, 0-9=48-57, A-Z=65-90
    _32,_33,_34,_35,_36,_37,_38,_39,_40,_41,_42,_43,_44,_45,_46,_47,
    _48,_49,_50,_51,_52,_53,_54,_55,_56,_57,_58,_59,_60,_61,_62,_63,
    _64,_65,_66,_67,_68,_69,_70,_71,_72,_73,_74,_75,_76,_77,_78,_79,
    _80,_81,_82,_83,_84,_85,_86,_87,_88,_89,_90,

    CHAR_COUNT
};

//*****************************************************************************
char menumap_main[20][20]=
{// 01  02  03  04  05  06  07  08  09  10  11  12  13  14  15  16  17  18  19  20
    BR ,BR ,BR ,BR ,BR ,BR ,BR ,BR ,BR ,BR ,BR ,BR ,BR ,BR ,BR ,BR ,BR ,BR ,BR ,BR , //01
    ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,DI ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES , //02
    ES ,ES ,RK ,RK ,ES ,ES ,ES ,RK ,ES ,ES ,ES ,RK ,RK ,ES ,RK ,ES ,RK ,ES ,DI ,ES , //03
    ES ,ES ,RK ,ES ,RK ,ES ,RK ,ES ,RK ,ES ,RK ,ES ,ES ,ES ,RK ,RK ,ES ,ES ,ES ,ES , //04
    ES ,ES ,RK ,RK ,ES ,ES ,RK ,ES ,RK ,ES ,RK ,ES ,ES ,ES ,RK ,RK ,ES ,ES ,ES ,ES , //05
    ES ,ES ,RK ,ES ,RK ,ES ,ES ,RK ,ES ,ES ,ES ,RK ,RK ,ES ,RK ,ES ,RK ,ES ,ES ,ES , //06
    DI ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES , //07
    ES ,ES ,ES ,ES ,RK ,RK ,ES ,ES ,RK ,ES ,RK ,ES ,RK ,RK ,ES ,ES ,ES ,ES ,ES ,ES , //08
    ES ,ES ,ES ,ES ,RK ,ES ,RK ,ES ,RK ,ES ,RK ,ES ,RK ,ES ,RK ,ES ,ES ,DI ,ES ,ES , //09
    ES ,ES ,ES ,ES ,RK ,RK ,ES ,ES ,RK ,ES ,RK ,ES ,RK ,ES ,RK ,ES ,ES ,ES ,ES ,ES , //10
    ES ,ES ,DI ,ES ,RK ,ES ,RK ,ES ,ES ,RK ,RK ,ES ,RK ,ES ,RK ,ES ,ES ,ES ,ES ,ES , //11
    ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,DI ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES , //12
    ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES , //13
    ES ,ES ,ES ,BS ,'P','L','A','Y',' ','C','A','V','E',' ','A',ES ,ES ,ES ,ES ,ES , //14
    ES ,ES ,ES ,BS ,'T','I','M','E','R',' ','O','N',' ',' ',' ',ES ,ES ,DI ,ES ,ES , //15
    ES ,DI ,ES ,BS ,'H','I','S','C','O','R','E','S',' ',' ',' ',ES ,ES ,ES ,ES ,ES , //16
    ES ,ES ,ES ,BS ,'A','B','O','U','T',' ',' ',' ',' ',DI ,' ',ES ,ES ,ES ,ES ,ES , //17
    ES ,ES ,ES ,BS ,'Q','U','I','T',' ',' ',' ',' ',' ',' ',' ',ES ,ES ,ES ,ES ,DI , //18
    ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES ,ES , //19
    BR ,BR ,BR ,BR ,BR ,BR ,BR ,BR ,BR ,BR ,BR ,BR ,BR ,BR ,BR ,BR ,BR ,BR ,BR ,BR };//20

//*****************************************************************************
#define nyb(_hi,_lo) ((_hi<<4)|(_lo&0x0f))
#define maprow(\
    m01,m02,m03,m04,m05,m06,m07,m08,m09,m10,\
    m11,m12,m13,m14,m15,m16,m17,m18,m19,m20,\
    m21,m22,m23,m24,m25,m26,m27,m28,m29,m30,\
    m31,m32,m33,m34,m35,m36,m37,m38,m39,m40)\
    nyb(m01,m02),nyb(m03,m04),nyb(m05,m06),nyb(m07,m08),nyb(m09,m10),\
    nyb(m11,m12),nyb(m13,m14),nyb(m15,m16),nyb(m17,m18),nyb(m19,m20),\
    nyb(m21,m22),nyb(m23,m24),nyb(m25,m26),nyb(m27,m28),nyb(m29,m30),\
    nyb(m31,m32),nyb(m33,m34),nyb(m35,m36),nyb(m37,m38),nyb(m39,m40)

char cavemap_A[]=
{//        01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40
    maprow(MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ES,ER,ER,DI,ER,RK,ES,ER,ER,ER,ER,ER,RK,ER,RK,ER,ER,ER,ER,ER,ER,ER,ES,ER,ER,ER,ER,RK,ER,ER,ER,ER,MP),
    maprow(MP,ER,RK,SP,RK,ER,ER,ER,ER,ER,ER,ES,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,DI,ER,ER,RK,ER,ER,ER,ER,ES,ER,ER,ER,ER,ER,ES,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ES,ER,ER,ES,ER,ER,ER,ER,ER,RK,ER,RK,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,MP),
    maprow(MP,RK,ER,ES,ES,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,RK,ER,ER,RK,ER,ER,ER,ER,RK,ER,ER,ER,RK,ER,ER,ER,ER,ER,MP),
    maprow(MP,RK,ER,RK,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,RK,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,RK,ER,RK,ES,ER,MP),
    maprow(MP,ER,ER,ER,RK,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,RK,ER,ES,RK,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,RK,RK,ER,MP),
    maprow(MP,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,ER,ER,ER,RK,ER,ER,RK,ER,MP),
    maprow(MP,ER,ES,ER,ER,ER,RK,ER,ER,DI,ER,ES,ER,ER,RK,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,DI,ER,RK,ES,ER,ER,ER,ER,ER,ER,RK,ER,MP),
    maprow(MP,ER,ER,DI,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ES,ER,ER,ER,ER,ER,ER,ER,ER,RK,ES,ES,RK,ER,ER,DI,ER,ER,ER,ER,RK,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,RK,ER,ER,RK,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,RK,ER,RK,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ES,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,RK,RK,ES,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,RK,ER,DI,ER,ER,ER,ER,ES,ER,MP),
    maprow(MP,ER,RK,ER,ER,ES,ER,ER,RK,ER,ES,ES,ER,ER,ER,ER,ER,RK,ER,RK,DI,ER,ER,DI,ER,ER,ER,ER,RK,ER,ER,ER,RK,ER,ER,DI,ER,RK,ER,MP),
    maprow(MP,ER,DI,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,RK,RK,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,DI,ER,ER,ER,ER,ER,RK,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,MP),
    maprow(MP,ES,ES,ER,ER,ER,ER,ER,ER,ER,ER,ER,ES,ER,ER,ER,DI,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,RK,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,RK,ES,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,RK,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,RK,ER,RK,ES,ER,ER,EX,MP),
    maprow(MP,ER,RK,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,RK,ER,ES,ES,ER,ER,ER,ER,DI,ER,ER,ER,RK,ER,RK,RK,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,RK,DI,ER,ER,ES,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,RK,ER,RK,DI,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ES,ER,ER,ES,ER,RK,ER,ER,RK,ER,RK,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,RK,DI,ER,ER,ER,ER,ER,ER,RK,ER,ER,RK,MP),
    maprow(MP,ER,DI,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ES,ER,ER,ER,ER,ER,ER,ER,ER,ER,ES,ER,RK,ER,ER,RK,ER,ER,ER,ER,RK,ER,ER,ER,RK,ER,MP),
    maprow(MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP)};

char cavemap_B[]=
{//        01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40
    maprow(MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP),
    maprow(MP,ER,RK,ER,ER,RK,ER,ER,BR,ER,RK,ER,ER,ER,DI,ER,BR,ER,ER,ER,ES,ER,RK,ER,BR,ES,ER,ER,ER,ER,ER,ER,BR,ER,ER,RK,RK,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,BR,ER,ER,ER,ER,ER,ER,ES,BR,ES,ES,ER,ES,ER,ER,ER,BR,ES,ER,ER,ES,ER,ER,ER,BR,ER,ER,ER,ER,ES,ER,MP),
    maprow(MP,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,RK,ES,RK,RK,ES,ES,ES,ES,ES,ES,RK,ES,ES,DI,ES,ES,ES,ES,ES,ES,ES,ES,RK,ES,MP),
    maprow(MP,DI,ER,ER,ER,ER,ER,ER,BR,ER,RK,ER,ER,ER,ER,RK,BR,ER,RK,ER,ES,ER,ER,ES,BR,ER,ER,RK,ER,ER,DI,ER,BR,ER,ER,RK,ER,RK,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,BR,ER,RK,ER,ER,ER,ER,RK,BR,ER,RK,ER,ES,ES,ER,ER,BR,ER,ER,ER,ER,ER,RK,ER,BR,ER,ER,ER,ES,ER,ER,MP),
    maprow(MP,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,ES,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,MP),
    maprow(MP,ER,ER,ER,ER,RK,RK,ER,BR,ER,ER,ES,ER,ER,ER,ER,BR,ER,ER,ER,ES,ER,ER,RK,BR,ER,ER,ER,ER,ES,ER,ER,BR,ER,ER,ER,ER,ER,ES,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,BR,ER,ER,ES,ER,ER,ER,ER,BR,ER,ER,ER,ES,ER,ER,ER,BR,ER,ER,ER,ER,RK,ER,ES,BR,ER,ER,ER,ER,ER,RK,MP),
    maprow(MP,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,RK,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,RK,ES,ES,ES,ES,ES,ES,ES,ES,RK,MP),
    maprow(MP,RK,ER,ER,RK,ER,ER,ER,BR,ER,ER,ER,ER,RK,ER,ER,BR,ER,ER,RK,ES,ER,ER,ER,BR,ER,ER,ER,ER,ER,ER,DI,BR,ES,ER,ER,ER,ER,ER,MP),
    maprow(MP,RK,ER,ER,ER,ER,RK,ER,BR,ER,ER,RK,ER,ER,RK,ER,BR,ER,ER,ER,ES,ER,ES,ES,BR,ER,ER,ER,ER,ER,ER,ER,BR,RK,ER,ER,ER,RK,ER,MP),
    maprow(MP,ER,RK,ER,ER,ER,ER,ER,BR,ER,ER,ER,RK,ER,ER,ER,BR,ER,ER,ER,ES,ER,RK,RK,BR,ER,ER,ER,ER,ER,ER,ER,BR,RK,RK,ER,ER,RK,ER,MP),
    maprow(MP,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,ES,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,MP),
    maprow(MP,RK,ER,ES,ES,BM,ER,ER,BR,ER,ER,ER,ER,RK,ER,RK,BR,ER,ER,ER,ES,ER,ER,ER,BR,ER,RK,DI,ER,ER,RK,ER,BR,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ES,ER,BR,ES,ER,ER,ER,ER,ER,ER,BR,ER,ER,ES,ES,ER,ER,ER,BR,ES,ER,ER,ES,ER,ER,ER,BR,ER,ES,ER,ES,ES,ER,MP),
    maprow(MP,ES,ES,ES,ES,ES,ES,ES,ES,RK,ES,ES,ES,ES,ES,ES,ES,ES,ES,DI,ES,ES,ES,ES,ES,ES,ES,ES,RK,ES,ES,ES,ES,ES,RK,ES,RK,RK,ES,MP),
    maprow(MP,DI,ER,ER,ES,ER,RK,ER,BR,RK,ER,ER,ER,ER,RK,ER,BR,ER,RK,ER,ES,ER,ER,RK,BR,ER,RK,ER,RK,ER,ER,ER,BR,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,RK,ER,BR,RK,ER,ER,DI,ER,ER,ER,BR,ER,ER,ER,ES,ES,ER,ER,BR,ER,ER,RK,ER,ER,ER,ER,BR,ER,ER,ER,ES,ES,ES,MP),
    maprow(MP,ER,DI,ER,ER,ER,RK,ES,BR,ER,ER,ES,ER,ER,ER,ER,BR,ER,SP,DI,RK,RK,ER,ER,BR,ER,ES,ER,ER,ER,ER,ER,BR,ER,ER,ER,RK,RK,RK,MP),
    maprow(MP,ER,RK,ER,ER,ER,ER,RK,BR,ER,ER,RK,ER,ER,RK,ER,BR,ER,EX,ER,RK,ER,ER,ER,BR,ER,ER,ER,ER,RK,ER,RK,BR,ER,ER,ER,ER,RK,ER,MP),
    maprow(MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP)};

char cavemap_C[]=
{//        01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40
    maprow(MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP),
    maprow(MP,RK,ER,BR,BR,ER,BR,RK,RK,ER,BR,ER,ER,ER,RK,BR,RK,ER,ER,RK,ER,ER,ER,ER,BR,ER,ER,ER,RK,ER,ER,ER,ER,ER,RK,BR,ER,DI,ER,MP),
    maprow(MP,ER,ER,SP,BR,ER,DI,ER,RK,ER,BR,ER,ER,ER,BR,BR,BR,ER,ER,BR,ER,RK,ER,ER,ER,ER,RK,ER,ER,RK,ER,RK,ER,ER,ER,BR,ER,BR,RK,MP),
    maprow(MP,ER,ER,ER,ER,BR,ER,ER,RK,DI,ER,ER,RK,ER,ER,ER,ER,BR,ER,ER,ER,ER,ER,RK,ER,BR,BR,RK,ER,ER,ER,ER,ER,ER,ER,BR,ER,BR,BR,MP),
    maprow(MP,DI,ER,BR,ER,ER,BR,RK,BR,RK,ER,ER,RK,ER,ER,ER,ER,BR,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,RK,ER,RK,RK,ER,ER,ER,ER,ER,ER,BR,MP),
    maprow(MP,RK,ER,BR,ER,ER,ER,BR,ER,ER,RK,ER,BR,BR,ER,ER,RK,ER,BR,BR,DI,ER,ER,ER,ER,ER,ER,ER,RK,ER,RK,RK,ER,ER,ER,ER,ER,ER,BR,MP),
    maprow(MP,RK,RK,ER,ER,RK,ER,ER,ER,ER,BR,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,RK,ER,RK,RK,ER,ER,ER,ER,ER,ER,RK,ER,ER,DI,BR,BR,ER,ER,MP),
    maprow(MP,ER,ER,RK,ER,BR,BR,ER,ER,RK,ER,RK,RK,ER,ER,ER,BR,ER,ER,ER,ER,RK,ER,RK,RK,ER,ER,ER,ER,ER,ER,BR,ER,ER,RK,ER,BR,ER,RK,MP),
    maprow(MP,ER,ER,BR,ER,ER,ER,DI,ER,ER,ER,ER,ER,ER,DI,ER,RK,ER,ER,BR,BR,RK,ER,ER,RK,ER,BR,ER,BR,RK,ER,ER,BR,RK,ER,ER,DI,ER,RK,MP),
    maprow(MP,RK,ER,RK,ER,ER,ER,ER,BR,ER,BR,BR,ER,ER,DI,ER,RK,ER,ER,BR,BR,RK,ER,ER,RK,ER,ER,DI,ER,BR,ER,ER,ER,BR,ER,ER,RK,ER,BR,MP),
    maprow(MP,ER,RK,ER,BR,BR,ER,ER,ER,ER,ER,RK,RK,BR,RK,ER,ER,DI,ER,BR,ER,BR,RK,ER,ER,BR,RK,ER,ER,ER,BR,RK,ER,ER,DI,ER,RK,ER,ER,MP),
    maprow(MP,BR,ER,BR,BR,ER,ER,ER,ER,ER,ER,RK,RK,BR,RK,ER,ER,RK,ER,BR,ER,BR,BR,ER,ER,ER,BR,ER,ER,RK,ER,BR,BR,ER,ER,RK,ER,BR,BR,MP),
    maprow(MP,ER,BR,ER,RK,ER,RK,ER,BR,ER,ER,ER,BR,BR,RK,ER,ER,RK,ER,ER,ER,ER,BR,ER,ER,ER,RK,ER,ER,ER,ER,ER,BR,BR,ER,RK,ER,BR,BR,MP),
    maprow(MP,ER,BR,ER,RK,ER,RK,ER,BR,ER,DI,ER,BR,ER,BR,RK,ER,ER,BR,RK,ER,ER,ER,ER,RK,ER,ER,RK,ER,RK,RK,ER,ER,ER,ER,BR,ER,ER,ER,MP),
    maprow(MP,BR,ER,ER,BR,RK,BR,RK,ER,ER,RK,ER,ER,ER,ER,BR,ER,ER,ER,DI,ER,ER,ER,BR,ER,RK,BR,ER,ER,ER,ER,ER,ER,BR,ER,BR,BR,ER,DI,MP),
    maprow(MP,BR,ER,ER,ER,BR,BR,RK,ER,ER,BR,ER,DI,ER,ER,ER,BR,RK,ER,ER,RK,ER,RK,ER,ER,ER,RK,ER,BR,RK,ER,ER,ER,ER,ER,ER,BR,ER,ER,MP),
    maprow(MP,BR,ER,DI,ER,ER,ER,ER,RK,ER,BR,BR,ER,ER,RK,ER,BR,BR,RK,ER,ER,ER,ER,ER,ER,ER,RK,ER,BR,RK,ER,ER,ER,ER,ER,ER,BR,ER,ER,MP),
    maprow(MP,ER,ER,RK,ER,ER,ER,ER,BR,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,RK,ER,RK,RK,ER,ER,ER,ER,ER,ER,BR,ER,ER,RK,ER,BR,ER,ER,ER,BR,MP),
    maprow(MP,RK,ER,BR,BR,ER,ER,RK,ER,BR,BR,ER,ER,ER,BR,ER,ER,ER,ER,RK,ER,RK,RK,ER,ER,ER,ER,ER,ER,BR,ER,ER,RK,DI,ER,ER,RK,ER,ER,EX),
    maprow(MP,BR,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,RK,ER,RK,DI,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,BR,BR,ER,ER,BR,RK,ER,ER,DI,ER,BR,ER,ER,MP),
    maprow(MP,RK,RK,ER,ER,ER,BR,ER,ER,ER,ER,ER,RK,ER,RK,DI,ER,ER,ER,ER,ER,ER,BR,ER,ER,RK,ER,BR,DI,ER,DI,ER,RK,BR,ER,RK,ER,ER,ER,MP),
    maprow(MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP)};

char cavemap_D[]=
{//        01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40
    maprow(MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP),
    maprow(MP,SP,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,RK,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ES,ES,ER,ER,ER,RK,ER,ER,ES,ES,ER,ER,ER,ER,ER,ER,ES,ES,ER,ER,ER,ER,ER,ER,ES,ES,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,BF,ES,ER,ER,ER,RK,ER,ER,BF,ES,ER,ER,ER,ER,ER,ER,BF,ES,ER,ER,ER,ER,ER,ER,BF,ES,ER,RK,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,RK,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,RK,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,RK,ER,RK,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,RK,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,RK,ER,RK,ER,ER,ER,ER,ER,ER,ER,EX,MP),
    maprow(MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP)};

char cavemap_E[]=
{//        01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40
    maprow(MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP),
    maprow(MP,SP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ES,ES,ES,ER,ER,ER,ER,ER,ES,ES,ES,ER,ER,ER,ER,ER,ES,ES,ES,ER,ER,ER,ER,ER,ES,ES,ES,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ES,ES,ES,ER,ER,ER,ER,ER,ES,ES,ES,ER,ER,ER,ER,ER,ES,ES,ES,ER,ER,ER,ER,ER,ES,ES,ES,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ES,DI,BM,ER,ER,ER,ER,ER,ES,DI,BM,ER,ER,ER,ER,ER,ES,DI,BM,ER,ER,ER,ER,ER,ES,DI,BM,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ES,ES,ES,ER,ER,ER,ER,ER,ES,ES,ES,ER,ER,ER,ER,ER,ES,ES,ES,ER,ER,ER,ER,ER,ES,ES,ES,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ES,ES,ES,ER,ER,ER,ER,ER,ES,ES,ES,ER,ER,ER,ER,ER,ES,ES,ES,ER,ER,ER,ER,ER,ES,ES,ES,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ES,DI,BM,ER,ER,ER,ER,ER,ES,DI,BM,ER,ER,ER,ER,ER,ES,DI,BM,ER,ER,ER,ER,ER,ES,DI,BM,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,EX),
    maprow(MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP)};

char cavemap_F[]=
{//        01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40
    maprow(MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP),
    maprow(MP,BR,BR,BR,BR,BR,BR,BR,BR,BR,ER,ER,ER,ER,RK,ER,RK,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,BR,BR,BR,BR,BR,BR,BR,BR,BR,MP),
    maprow(MP,BR,BM,ES,ES,ES,ES,ES,ES,ES,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,RK,ER,ER,ER,ES,BM,ES,ES,ES,ES,ES,ES,BR,MP),
    maprow(MP,BR,ES,DI,ES,ES,ES,ES,ES,ES,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,RK,ER,ER,ES,ES,ES,ES,ES,ES,DI,ES,BR,MP),
    maprow(MP,BR,BR,BR,BR,BR,BR,BR,BR,BR,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,RK,ER,BR,BR,BR,BR,BR,BR,BR,BR,BR,MP),
    maprow(MP,BR,BM,ES,ES,ES,ES,ES,ES,ES,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,RK,ER,ES,BM,ES,ES,ES,ES,ES,ES,BR,MP),
    maprow(MP,BR,ES,DI,ES,ES,ES,ES,ES,ES,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,RK,ER,RK,RK,ER,ER,ER,ER,ER,ES,ES,ES,ES,ES,ES,DI,ES,BR,MP),
    maprow(MP,BR,BR,BR,BR,BR,BR,BR,BR,BR,ER,RK,RK,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,RK,RK,ER,ER,ER,ER,ER,BR,BR,BR,BR,BR,BR,BR,BR,BR,MP),
    maprow(MP,BR,BM,ES,ES,ES,ES,ES,ES,ES,ER,ER,ER,ER,RK,ER,RK,ER,ER,ER,ER,RK,ER,ER,RK,ER,ER,ER,ER,ER,ES,BM,ES,ES,ES,ES,ES,ES,BR,MP),
    maprow(MP,BR,ES,DI,ES,ES,ES,ES,ES,ES,ER,ER,ER,ER,RK,ER,RK,ER,ER,ER,ER,RK,ER,ER,RK,ER,ER,RK,ER,ER,ES,ES,ES,ES,ES,ES,DI,ES,BR,MP),
    maprow(MP,BR,BR,BR,BR,BR,BR,BR,BR,BR,ER,RK,RK,ER,RK,ER,ER,RK,ER,ER,ER,ER,RK,ER,ER,ER,RK,ER,ER,ER,BR,BR,BR,BR,BR,BR,BR,BR,BR,MP),
    maprow(MP,BR,BM,ES,ES,ES,ES,ES,ES,ES,ER,RK,RK,ER,RK,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ES,BM,ES,ES,ES,ES,ES,ES,BR,MP),
    maprow(MP,BR,ES,DI,ES,ES,ES,ES,ES,ES,ER,ER,ER,ER,RK,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ES,ES,ES,ES,ES,ES,DI,ES,BR,MP),
    maprow(MP,BR,BR,BR,BR,BR,BR,BR,BR,BR,ER,ER,ER,ER,ER,RK,ER,ER,ER,RK,ER,ER,ER,ER,RK,ER,ER,RK,ER,ER,BR,BR,BR,BR,BR,BR,BR,BR,BR,MP),
    maprow(MP,ER,ER,ER,ER,RK,ER,RK,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,RK,ER,ER,ER,ER,RK,ER,ER,RK,ER,RK,ER,ER,ER,RK,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,RK,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,RK,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,RK,ER,RK,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,RK,ER,SP,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,RK,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,RK,ER,ER,RK,ER,EX,MP),
    maprow(MP,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,RK,ER,RK,RK,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,RK,ER,ER,ER,ER,MP),
    maprow(MP,RK,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,RK,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,RK,ER,RK,ER,RK,ER,ER,RK,ER,ER,ER,MP),
    maprow(MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP)};

char cavemap_G[]=
{//        01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40
    maprow(MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP),
    maprow(MP,ER,ES,ER,ER,ES,ER,RK,RK,ER,ER,ER,ER,ER,ES,ER,ER,ES,ER,ES,SP,ER,ER,ER,ER,ES,RK,ES,ES,ES,ER,ER,RK,ER,ES,ER,ES,ES,ER,MP),
    maprow(MP,ES,ER,ER,RK,ER,ES,ER,ER,ES,ER,ES,ES,ER,ER,ER,ER,ES,ER,RK,ER,ES,ER,ES,ER,ER,ER,ES,RK,RK,ER,ER,RK,ER,DI,ER,ER,ES,ER,MP),
    maprow(MP,RK,ER,ER,ER,ER,ER,ES,ES,ER,ES,ER,ES,ES,ER,ER,ER,RK,ER,RK,ER,RK,ER,ES,ER,ER,ER,RK,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,MP),
    maprow(MP,ER,RK,ER,DI,ER,ER,ER,ES,ER,ES,BM,ER,ER,ER,ER,ER,ER,ES,ER,ER,RK,RK,ER,ER,ES,ER,ER,ER,ER,ES,ER,ES,ER,ER,ER,ES,ER,ES,MP),
    maprow(MP,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,ER,ES,ER,ES,ER,ER,ES,ES,RK,RK,ER,ER,ES,ER,ER,ER,ER,ES,ER,ER,ER,RK,ER,ER,ER,ER,EX),
    maprow(MP,RK,ER,ES,ES,ER,ER,ER,ER,ER,ER,ES,ER,ER,RK,ER,ES,ER,ER,ER,ES,ER,ER,ES,ER,RK,ES,ER,ER,RK,ER,ES,ES,BM,ER,ER,ER,ER,ER,MP),
    maprow(MP,RK,ER,RK,ES,ER,ER,ER,ER,ER,ER,ES,ER,ER,RK,RK,ER,ER,ES,ER,ER,ER,ER,RK,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ES,ER,RK,ES,ER,MP),
    maprow(MP,ER,ER,ER,ES,ER,ER,RK,ES,ES,ER,ER,ER,ES,ER,ER,ES,ER,RK,ES,ER,ER,RK,ER,ES,ES,ER,ER,ER,ES,ER,ER,ER,ER,ES,ER,RK,RK,ER,MP),
    maprow(MP,ER,ER,ER,RK,ER,ER,RK,ER,ES,ER,ES,ER,ER,ER,ER,RK,ER,ER,ER,BM,ER,ER,ER,ER,ER,ER,RK,ER,RK,ER,ER,ES,ES,ES,ER,ER,RK,ER,MP),
    maprow(MP,ES,ES,ER,ER,ES,ES,ER,ER,ER,ER,ES,ER,ER,RK,ER,RK,ER,ER,ER,ER,ES,ER,ES,ES,ER,ER,ER,ER,ER,ER,ER,ES,RK,ES,ER,ER,RK,ER,MP),
    maprow(MP,ER,ES,ER,ER,ER,RK,ER,ER,ES,ER,RK,ES,ER,ER,ES,ER,ES,ES,ER,ER,ER,ER,ER,ES,ES,ES,ES,ER,ER,ES,ER,ES,ER,RK,ES,ER,ER,ES,MP),
    maprow(MP,ER,ER,ES,ES,ER,ER,RK,ER,RK,ER,ER,ER,ER,ES,ER,ES,ES,ER,ER,ER,ER,ER,ER,RK,ES,ES,ES,ER,ER,RK,ER,ES,ER,DI,RK,ER,ER,ER,MP),
    maprow(MP,ER,RK,ER,DI,ES,ER,ER,ES,ER,ES,ES,ER,ER,ER,ES,ER,ES,ER,ES,ER,ES,ER,ER,ER,RK,RK,RK,ER,ER,RK,ER,ER,ER,ER,RK,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ES,ES,ER,ES,ER,ES,ES,ER,ER,ER,ES,ER,RK,ER,ES,ER,ES,ER,ES,ER,RK,ER,RK,ES,ER,ER,ES,RK,ER,ER,ER,ER,ES,ER,MP),
    maprow(MP,ER,ES,ES,ER,ER,ER,ER,RK,ER,ER,ER,ER,ES,ES,ER,ER,RK,ES,RK,RK,ER,ER,ES,ER,ER,ER,ER,RK,ER,ER,ER,RK,ER,ER,ER,ES,ER,RK,MP),
    maprow(MP,ER,ER,ER,ER,ER,ES,ER,RK,ES,ER,RK,ES,ER,ES,ER,ER,ER,RK,RK,RK,ER,ER,RK,ER,RK,ER,ER,ER,ES,ES,ER,ER,RK,ER,RK,ER,ER,ER,MP),
    maprow(MP,ES,ES,ER,ER,ER,ER,ER,ER,ES,ER,ER,RK,ER,ES,ER,ES,ER,ER,ER,ER,ES,ER,RK,ES,ER,ER,RK,ER,ES,RK,RK,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,RK,ES,ER,ER,ER,ER,ER,ER,ES,ER,ER,RK,RK,ER,ER,RK,ER,ER,ER,GS,ES,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,RK,ER,RK,RK,ER,ER,ER,MP),
    maprow(MP,ER,ES,ER,ER,RK,ER,ES,ER,ER,ER,ES,ER,ER,ES,ER,RK,ES,ER,GS,GS,ER,ES,ES,ER,ER,ER,ES,ER,ER,ER,ER,RK,ER,RK,RK,ER,ER,ER,MP),
    maprow(MP,ER,RK,ER,DI,RK,BM,ER,ER,RK,ER,ER,ER,ER,RK,ER,ER,ER,RK,GS,GS,ER,ER,ER,ER,RK,ER,RK,BM,ER,ER,ER,ER,ER,DI,RK,ER,ER,ER,MP),
    maprow(MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP)};

char cavemap_H[]=
{//        01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40
    maprow(MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP),
    maprow(MP,ES,ER,ES,RK,ER,ER,ES,ER,ES,ER,ER,ES,ER,ER,RK,ER,ES,ER,ER,SP,ES,ER,ER,ES,ER,ES,ES,ER,ER,RK,ER,ES,RK,ER,ER,ER,ES,ER,MP),
    maprow(MP,ER,RK,ER,RK,ES,ER,ER,ER,ER,ER,ER,ES,ER,ER,RK,ER,ER,ER,ES,ER,ER,ER,ER,RK,ER,ER,ER,RK,ER,ER,ER,ER,ER,DI,RK,ER,RK,ER,MP),
    maprow(EX,ES,RK,ER,ER,RK,ER,ER,ER,ES,BM,ER,ER,ER,RK,ER,ER,RK,ER,RK,ER,ER,RK,ER,RK,ER,ER,ER,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,MP),
    maprow(MP,ER,ER,ER,DI,RK,ER,ER,RK,ER,ES,ES,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ES,ER,ER,ER,ER,ER,ER,ER,ER,ES,ES,ES,ES,ER,ER,ES,MP),
    maprow(MP,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,ER,ER,RK,ER,RK,ER,ER,ER,ER,ES,ER,ES,ES,ER,ER,ER,ER,ER,ER,RK,ES,ES,ES,ER,ER,RK,MP),
    maprow(MP,ER,ES,ES,ER,ER,ER,ES,ER,ER,ES,ER,ES,ES,ER,ER,ES,ER,ES,ES,ER,ER,ER,ER,ES,RK,RK,ES,ER,ER,ER,ER,ER,RK,RK,RK,ER,ER,RK,MP),
    maprow(MP,ER,ER,ER,ES,ES,ER,ER,ER,ES,ES,ES,ER,ES,ER,ER,ES,ER,ES,ES,ER,ER,ER,ER,ER,RK,ER,ES,RK,ER,ER,ES,ER,RK,ER,RK,ES,ER,ER,MP),
    maprow(MP,ER,ER,RK,ER,RK,ER,ER,RK,ES,ES,ES,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ES,ER,ER,ER,ES,RK,RK,ER,ER,RK,ER,ER,ER,ER,RK,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,RK,ES,ER,ES,ES,ES,ER,ER,ES,ER,RK,ES,ES,RK,ES,ER,ES,ER,ER,ER,RK,RK,RK,ER,ER,RK,ER,RK,ER,ER,ER,ER,ES,MP),
    maprow(MP,RK,ER,RK,ER,ER,ER,ES,ER,ES,ES,ER,ER,ER,ER,ER,ER,BM,ER,ER,RK,ER,ER,ER,ES,ER,ER,ER,ER,RK,ER,ER,ER,ER,DI,RK,ER,ES,RK,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,RK,ER,RK,ES,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,RK,ER,ER,RK,ER,ER,ER,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,MP),
    maprow(MP,ER,RK,RK,ER,ER,ER,ER,ER,ER,RK,ER,ER,RK,ER,ES,ER,ER,ER,ES,ER,ER,ES,ER,RK,ES,ER,ER,RK,ER,ES,ES,ER,ER,ER,ES,RK,ER,ER,MP),
    maprow(MP,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,RK,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ES,ER,RK,RK,ER,ER,MP),
    maprow(MP,ER,ER,ES,ER,ER,ER,ES,ES,ER,ER,ER,DI,ER,ER,RK,ER,ES,ER,ER,RK,ER,RK,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ES,ER,RK,RK,ER,ER,MP),
    maprow(MP,ER,ER,RK,ER,ER,RK,ER,ES,ER,ES,ER,ER,ER,MB,MB,MB,MB,MB,MB,MB,MB,ER,ER,ER,ER,ER,ER,ER,ER,ER,ES,ES,ES,ER,ER,RK,ER,ER,MP),
    maprow(MP,ES,ER,ER,ES,ES,ER,ER,ER,ER,RK,ER,ER,RK,ES,RK,ER,ER,ER,DI,ES,ER,ER,ES,ER,ER,ER,ER,ER,ER,ER,ES,RK,ES,ER,ER,RK,ER,ER,MP),
    maprow(MP,RK,ER,ER,ER,RK,ER,ER,RK,ER,RK,ER,ER,ER,RK,ER,ES,ES,ER,ER,ER,ER,ER,ES,ES,ES,ES,ER,ER,ES,ER,ES,ER,RK,ES,ER,ER,ES,RK,MP),
    maprow(MP,ER,ES,ES,ER,ER,BM,ER,RK,ER,ER,ER,ER,ES,ER,ES,ES,ER,ER,ER,ER,ER,ER,RK,RK,ES,ES,ER,ER,RK,ER,ER,ER,RK,RK,ER,ER,ER,RK,MP),
    maprow(MP,RK,ER,RK,ES,ER,ER,RK,ER,ES,ES,ER,ER,ER,ER,ER,RK,ER,RK,ER,ES,ER,ER,ER,RK,RK,RK,ER,ER,RK,ER,ER,ER,ER,RK,ER,ER,ER,RK,MP),
    maprow(MP,ER,ER,ER,ES,ES,ER,RK,ER,RK,ES,ER,ER,ER,ER,ER,RK,ER,RK,ER,ER,ER,ER,ER,RK,RK,RK,ER,ER,RK,ER,RK,ER,ER,ER,ER,RK,ER,ER,MP),
    maprow(MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP)};

char cavemap_I[]=
{//        01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40
    maprow(MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP),
    maprow(MP,DI,DI,DI,RK,RK,DI,DI,RK,DI,DI,RK,ER,RK,RK,RK,RK,DI,RK,DI,DI,ER,DI,DI,RK,DI,DI,RK,DI,DI,DI,DI,RK,RK,DI,RK,DI,RK,RK,MP),
    maprow(MP,DI,RK,RK,DI,DI,DI,RK,RK,RK,DI,RK,DI,DI,RK,RK,RK,RK,RK,RK,DI,RK,RK,DI,ER,DI,RK,DI,RK,RK,RK,RK,DI,RK,DI,DI,RK,RK,DI,MP),
    maprow(MP,DI,DI,RK,RK,RK,RK,RK,RK,RK,DI,RK,DI,DI,RK,RK,ER,RK,RK,RK,DI,RK,RK,DI,DI,DI,RK,DI,RK,ER,RK,RK,DI,RK,RK,RK,DI,DI,RK,MP),
    maprow(MP,RK,RK,DI,RK,DI,DI,RK,RK,RK,RK,RK,RK,DI,RK,RK,DI,DI,DI,ER,ER,DI,DI,RK,RK,DI,RK,DI,DI,RK,RK,DI,RK,DI,DI,ER,RK,RK,DI,MP),
    maprow(MP,RK,RK,DI,RK,DI,DI,RK,RK,RK,RK,RK,RK,DI,RK,RK,DI,ER,DI,RK,DI,RK,RK,RK,RK,DI,RK,DI,RK,DI,RK,RK,DI,DI,RK,RK,DI,RK,DI,MP),
    maprow(MP,DI,DI,DI,RK,RK,DI,RK,DI,ER,DI,DI,RK,RK,DI,DI,RK,RK,DI,DI,DI,RK,RK,DI,RK,DI,RK,RK,RK,ER,DI,RK,DI,DI,RK,RK,DI,RK,DI,MP),
    maprow(MP,RK,RK,RK,RK,RK,DI,RK,RK,DI,DI,DI,DI,ER,ER,RK,RK,RK,DI,RK,DI,DI,ER,RK,DI,RK,DI,DI,RK,ER,RK,RK,DI,DI,DI,DI,DI,DI,RK,MP),
    maprow(MP,DI,RK,DI,DI,BR,BR,BR,BR,BR,BR,BR,ER,BR,BR,BR,BR,BR,DI,RK,RK,RK,RK,DI,RK,BR,BR,BR,BR,BR,BR,ER,BR,BR,BR,BR,BR,BR,RK,MP),
    maprow(MP,DI,ER,DI,DI,BR,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,BR,RK,DI,DI,RK,RK,RK,DI,BR,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,BR,RK,MP),
    maprow(MP,DI,RK,DI,RK,BR,ES,SP,EX,ES,ES,ES,ES,ES,ES,ES,ES,BR,RK,DI,DI,RK,RK,RK,DI,BR,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,BR,RK,MP),
    maprow(MP,DI,RK,RK,DI,BR,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,BR,RK,ER,RK,RK,DI,DI,RK,BR,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,BR,RK,MP),
    maprow(MP,DI,RK,RK,DI,BR,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,BR,DI,DI,DI,DI,RK,DI,RK,BR,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,BR,DI,MP),
    maprow(MP,RK,DI,DI,DI,BR,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,BR,DI,RK,RK,DI,ER,DI,RK,BR,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,BR,DI,MP),
    maprow(MP,RK,RK,RK,RK,BR,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,BR,DI,RK,RK,DI,DI,RK,RK,BR,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,BR,RK,MP),
    maprow(MP,DI,RK,DI,DI,BR,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,BR,ER,RK,DI,RK,RK,DI,RK,BR,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,BR,RK,MP),
    maprow(MP,DI,RK,DI,DI,BR,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,BR,BR,BR,BR,BR,BR,BR,BR,BR,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,BR,RK,MP),
    maprow(MP,RK,RK,RK,DI,BR,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,BR,RK,MP),
    maprow(MP,RK,RK,RK,DI,BR,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,BR,DI,DI,ER,RK,DI,RK,DI,BR,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,BR,RK,MP),
    maprow(MP,DI,DI,RK,RK,BR,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,BR,RK,RK,RK,DI,RK,DI,DI,BR,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,ES,BR,RK,MP),
    maprow(MP,DI,DI,ER,ER,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,DI,RK,RK,RK,DI,DI,DI,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,DI,MP),
    maprow(MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP)};

char cavemap_J[]=
{//        01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40
    maprow(MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,SP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,ES,BR,BR,BR,BR,BR,BR,BR,BR,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,BR,ER,ER,ER,ER,DI,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,DI,BR,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,BR,ER,BR,ES,BR,ER,BR,BR,BR,BR,BR,BR,ES,BR,BR,BR,BR,BR,BR,ER,BR,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,BR,ER,BR,BM,BR,ER,BR,DI,ER,ER,ER,ER,ER,ER,ER,ER,ER,DI,BR,ER,BR,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,BR,ER,BR,BM,BR,ER,BR,ER,BR,BR,BR,BR,ES,BR,BR,BR,BR,ER,BR,ER,BR,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,BR,ER,BR,BM,BR,ER,BR,ER,BR,DI,ER,ER,ER,ER,ER,DI,BR,ER,BR,ER,BR,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,BR,ER,BR,BM,BR,ER,BR,ER,BR,ER,BR,BR,BM,BR,BR,ER,BR,ER,BR,ER,BR,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,BR,ER,BR,BM,BR,BM,BR,BM,BR,BM,BR,BM,BM,ES,BR,BM,BR,BM,BR,BM,BR,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,BR,ER,BR,BM,BR,ES,BR,ES,BR,ES,BR,ES,ES,ES,BR,ES,BR,ES,BR,ES,BR,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,BR,ER,BR,BM,BR,ES,BR,ES,BR,ES,BR,ES,ES,ES,BR,ES,BR,ES,BR,ES,BR,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,BR,ER,BR,BM,BR,ER,BR,ER,BR,ER,BR,BR,BR,BR,BR,ER,BR,ER,BR,ER,BR,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,BR,ER,BR,BM,BR,ER,BR,ER,BR,DI,ER,ER,ER,ER,ER,DI,BR,ER,BR,ER,BR,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,BR,ER,BR,DI,BR,ER,BR,ER,BR,BR,BR,BR,BR,BR,BR,BR,BR,ER,BR,ER,BR,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,BR,ER,BR,DI,BR,ER,BR,DI,ER,ER,ER,ER,ER,ER,ER,ER,ER,DI,BR,ER,BR,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,BR,ER,BR,DI,BR,ER,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,ER,BR,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,BR,ER,BR,DI,BR,DI,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,DI,BR,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,EX),
    maprow(MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP)};

char cavemap_K[]=
{//        01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40
    maprow(MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP),
    maprow(MP,RK,ER,RK,ES,ER,RK,RK,RK,ER,BR,ER,ER,ER,DI,ES,RK,ER,ER,RK,BR,ER,ER,ER,DI,ER,ER,ER,RK,ER,BR,ER,ER,ER,DI,RK,ER,RK,ER,MP),
    maprow(MP,ER,ER,ER,DI,ER,RK,ER,RK,ER,BR,ER,ER,ER,RK,RK,RK,ER,ER,RK,BR,RK,ER,ER,ER,ER,RK,ER,ER,RK,BR,RK,ER,ER,ER,RK,ER,RK,ES,MP),
    maprow(MP,ER,ER,ER,ER,ES,ER,ER,RK,RK,BR,ER,ES,ER,ER,ER,ER,ES,ER,ER,BR,ER,ER,RK,ER,ES,RK,RK,ER,ER,BR,ER,ER,ER,ER,RK,ER,RK,RK,MP),
    maprow(MP,RK,ER,ES,ER,ER,ES,RK,RK,RK,BR,ER,RK,ER,ER,ER,ER,RK,ER,ER,BR,RK,ER,ER,ER,ER,ER,ER,RK,ER,BR,RK,ER,ER,ER,ER,ER,ER,RK,MP),
    maprow(MP,RK,ER,RK,ER,ER,ER,RK,ER,ER,BR,ER,RK,ES,ER,ER,RK,ER,RK,RK,BR,ER,ER,ER,ER,ER,ER,ER,RK,ER,BR,RK,ER,ER,ER,ER,ER,ER,RK,MP),
    maprow(MP,RK,RK,ER,ER,RK,ER,ER,ER,ER,BR,ER,ER,ER,RK,ER,ER,ER,ER,ER,BR,RK,ER,RK,RK,ER,ER,ER,ER,ER,BR,RK,ER,ER,ES,ES,ES,ER,ER,MP),
    maprow(MP,ER,ER,RK,ER,RK,RK,ER,ER,RK,BR,RK,RK,ER,ER,ER,RK,ER,ER,ER,BR,RK,ER,RK,RK,ER,ER,ER,ER,ER,BR,RK,ER,ER,RK,ES,ES,ES,BR,MP),
    maprow(MP,ER,ER,RK,ER,ER,ER,RK,ER,ER,BR,ER,ER,ER,RK,ER,RK,ER,ER,RK,BR,RK,ER,ER,RK,ER,ES,ER,RK,RK,BR,ER,ES,RK,ER,ES,ES,ES,ES,MP),
    maprow(MP,RK,ER,RK,ER,BR,BR,BR,BR,BR,BR,BR,BR,BR,BM,BR,BR,BR,BR,BR,BR,BR,BR,BR,RK,BR,BR,BR,BR,BR,BR,BR,BR,BR,ER,RK,ES,ES,RK,MP),
    maprow(MP,ER,RK,ER,ES,ES,ER,ER,ER,ER,ER,RK,RK,RK,RK,ER,ER,RK,ER,ES,ER,ES,ES,ER,ER,RK,RK,ER,ER,ER,ES,RK,ER,ER,RK,BR,RK,ER,ER,MP),
    maprow(MP,RK,ER,RK,ES,ER,ER,ER,ER,ER,ER,RK,RK,RK,RK,ER,ER,RK,ER,RK,ER,RK,RK,ER,ER,ER,RK,ER,ER,RK,ER,RK,RK,ER,BR,RK,ER,RK,RK,MP),
    maprow(MP,ER,ES,ER,RK,ER,RK,ER,ES,BR,ER,ER,RK,RK,RK,ER,ER,RK,ER,ER,ER,ER,RK,ER,ER,ER,RK,ER,ER,ER,ER,ER,RK,BR,ER,RK,ER,RK,RK,MP),
    maprow(MP,ER,ES,ER,RK,ER,RK,ER,ES,BR,ES,ER,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,RK,RK,ER,BR,ER,ER,RK,ER,ER,ER,MP),
    maprow(MP,ES,ER,ER,RK,RK,RK,RK,ER,BR,RK,ER,ER,ER,ER,RK,ER,ER,ER,RK,ER,ER,ER,ES,ER,RK,RK,ER,ER,ER,ER,BR,ER,RK,ER,RK,ES,ER,RK,MP),
    maprow(MP,RK,ER,ER,ER,RK,RK,RK,ER,BR,RK,ER,RK,ER,ER,ER,ES,RK,ER,ER,RK,ER,RK,ER,ER,ER,RK,ER,RK,RK,BR,ER,ER,ER,ER,ER,RK,ER,EX,MP),
    maprow(MP,RK,ER,RK,ER,ER,ER,ER,ES,BR,RK,ES,ER,ER,RK,ER,RK,RK,RK,ER,ER,ER,ER,ER,ER,ER,RK,ER,RK,BR,ER,ER,ER,ER,ER,ER,RK,ER,ER,MP),
    maprow(MP,ER,ER,RK,ER,ER,ER,ER,RK,BR,ER,ER,RK,ER,ER,ER,ER,ER,ER,RK,ER,RK,RK,ER,ER,ER,ER,ER,BR,RK,ER,ER,RK,ER,RK,ER,ER,ER,RK,MP),
    maprow(MP,RK,ER,RK,RK,ER,ER,RK,ER,BR,RK,ER,ER,ER,RK,ER,ER,ER,ER,RK,SP,RK,RK,ER,ER,ER,ER,ER,ER,RK,ER,ER,RK,BM,ER,ER,RK,ER,ER,MP),
    maprow(MP,RK,ER,ER,ER,RK,ER,ER,ER,BR,ER,ER,RK,ER,RK,BM,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ES,ES,ER,ER,BM,RK,ER,ER,RK,DI,RK,ER,ER,MP),
    maprow(MP,RK,RK,ER,DI,ER,ES,ER,ER,BR,ER,ER,RK,ER,RK,RK,ER,ER,ER,ER,ER,ER,RK,ER,ER,RK,ER,RK,RK,ER,ES,ER,RK,RK,ER,RK,ER,ER,ER,MP),
    maprow(MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP)};

char cavemap_L[]=
{//        01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40
    maprow(MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP),
    maprow(MP,RK,ER,ES,ER,ER,ER,RK,RK,ER,ER,ER,ER,ER,RK,ER,ES,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,RK,ER,ER,DI,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,DI,ER,RK,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,RK,ER,ER,ER,ER,RK,ER,ER,RK,ER,ER,ER,ER,ER,ES,ER,ER,ES,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,RK,DI,BR,ER,RK,ER,BR,ER,ER,ES,BR,ER,ER,ER,BR,RK,ER,ER,ER,RK,ER,ER,ER,ES,ES,ER,ER,ES,ER,ER,RK,MP),
    maprow(MP,DI,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,ER,BR,ER,ER,ER,BR,ER,ER,RK,BR,ER,ER,ER,ER,ER,RK,ER,ER,ES,BM,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,RK,ER,ER,ER,ER,ER,ER,ER,ER,BR,ER,ER,ER,BR,ER,RK,ER,BR,ES,DI,ER,BR,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,RK,RK,ER,ER,RK,ER,ER,ER,ER,BR,ER,ER,ER,BR,ER,ER,ER,BR,ER,ER,ES,BR,RK,RK,ER,ER,ER,ER,ER,ER,RK,ER,ER,DI,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,RK,ER,ER,ER,ER,ER,RK,BR,RK,RK,ER,BR,ER,ER,ER,BR,ER,ER,RK,BR,RK,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,RK,MP),
    maprow(MP,ER,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,ER,BR,ER,RK,ER,BR,ES,ER,RK,BR,ER,RK,ER,ER,ER,ER,RK,ER,ES,ES,ER,ER,ER,DI,ER,RK,MP),
    maprow(MP,RK,ER,RK,ER,ER,ER,ER,ER,ER,BR,ER,ER,ER,BR,ER,RK,ER,BR,ER,ER,RK,BR,ER,RK,ER,ER,DI,ER,ER,ER,ES,BM,ER,ER,ER,RK,ER,ER,MP),
    maprow(MP,ER,RK,ER,ER,ER,ER,ER,ER,ER,BR,RK,ES,ES,BR,ER,ER,DI,BR,ER,ER,RK,BR,ER,ER,ER,ES,ER,ER,ER,ER,ER,ER,ER,DI,ER,RK,ER,ER,MP),
    maprow(MP,ES,ER,ER,ER,ER,ER,ER,ER,ER,BR,RK,RK,RK,BR,ER,ER,RK,BR,ER,ER,ER,BR,ER,ER,ER,RK,ER,ER,RK,ER,ER,ER,ER,ER,RK,ER,ER,ER,MP),
    maprow(MP,ER,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,RK,BR,ER,ER,ER,BR,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ES,ES,MP),
    maprow(MP,ER,ER,ER,RK,ER,ES,ER,ER,ER,BR,ER,ER,ER,BR,RK,ER,ER,BR,RK,ER,ER,BR,ER,RK,ER,ER,RK,ER,RK,ER,ES,ES,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ES,RK,RK,ER,ER,BR,ER,ER,ER,BR,ER,ER,ER,BR,DI,ER,ER,BR,ER,ER,RK,ES,ER,ER,ER,ER,ES,BM,ER,ER,ER,ER,ER,DI,MP),
    maprow(MP,ER,ER,ER,ER,RK,ER,RK,ER,ER,BR,ER,DI,ER,ER,ER,ER,RK,BR,ER,RK,ER,BR,ER,ER,ER,RK,ER,ES,ES,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,ER,BR,ER,ER,ER,BR,ER,ER,ER,BR,ER,ER,ER,RK,ER,RK,ES,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,RK,ER,ER,ER,ER,ER,ER,BR,ER,RK,ER,BR,ER,ER,ER,BR,RK,ER,RK,BR,ER,ER,ER,ER,ER,ER,RK,ER,ER,RK,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,RK,ER,SP,ER,ER,ER,RK,ER,ES,BR,ER,ER,ER,BR,ER,ER,ER,BR,RK,ER,RK,BR,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,DI,ER,ER,ES,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,RK,ER,ER,ER,ER,BR,ER,RK,ER,BR,DI,ER,ER,BR,ER,ER,ER,BR,ER,ER,ER,ER,ES,ER,ER,ER,RK,ER,ER,DI,ER,RK,ER,ER,MP),
    maprow(MP,RK,RK,ER,ER,ER,ER,ER,ER,ER,BR,ER,RK,ER,BR,DI,ER,ER,BR,ER,ER,ER,BR,ER,ER,RK,ER,ER,DI,ER,DI,ER,RK,ER,ER,RK,ER,ER,ER,EX),
    maprow(MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP)};

char cavemap_M[]=
{//        01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40
    maprow(MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP),
    maprow(MP,RK,ER,ER,ER,ER,ER,RK,RK,ER,EX,ER,ER,ER,ER,ER,RK,ER,SP,RK,GS,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,RK,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,RK,ER,ER,RK,ER,RK,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,RK,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,RK,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,RK,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,RK,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,RK,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,RK,ER,RK,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,RK,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,RK,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,RK,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,RK,ER,RK,ER,ER,ER,ER,RK,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,RK,ER,ER,MP),
    maprow(MP,RK,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,RK,ER,RK,ER,ER,RK,ER,ER,ER,ER,RK,ER,ER,ER,RK,ER,ER,ER,ER,RK,ER,ER,RK,ER,RK,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,RK,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,RK,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,RK,ER,RK,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,RK,ER,RK,ER,ER,ER,RK,ER,ER,ER,ER,RK,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,RK,ER,RK,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,RK,ER,ER,ER,ER,RK,ER,ER,RK,ER,RK,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,RK,ER,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,RK,ER,BF,BF,BF,BF,BF,BF,BF,BF,BF,BF,BF,BF,BF,BF,BF,BF,BF,BF,BF,BF,BF,BF,BF,BF,BF,BF,BF,BF,BF,BF,ER,ER,ER,ER,MP),
    maprow(MP,RK,ER,ER,ER,RK,RK,RK,RK,RK,RK,RK,RK,RK,RK,RK,RK,RK,RK,RK,RK,RK,RK,RK,RK,RK,RK,RK,RK,RK,RK,RK,RK,RK,RK,ER,RK,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,MP),
    maprow(MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP)};

char cavemap_N[]=
{//        01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40
    maprow(MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP),
    maprow(MP,ER,ER,SP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,BM,ER,BM,ER,BM,ER,BM,ER,BM,ER,BM,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,RK,ER,RK,ER,RK,ER,RK,ER,RK,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ES,ER,ES,ER,ES,ER,ES,ER,ES,ER,ES,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ES,ER,ES,ER,ES,ER,ES,ER,ES,ER,ES,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ES,ER,ES,ER,ES,ER,ES,ER,ES,ER,ES,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ES,ER,ES,ER,ES,ER,ES,ER,ES,ER,BF,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ES,ER,ES,ER,ES,ER,ES,ER,ES,ER,ES,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ES,ER,ES,ER,ES,ER,ES,ER,BF,ER,ES,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ES,ER,ES,ER,ES,ER,ES,ER,ES,ER,ES,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ES,ER,ES,ER,ES,ER,BF,ER,ES,ER,ES,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ES,ER,ES,ER,ES,ER,ES,ER,ES,ER,ES,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,BF,ER,ES,ER,BF,ER,ES,ER,ES,ER,ES,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,EX),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ES,ER,BF,ER,ES,ER,ES,ER,ES,ER,ES,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP)};

char cavemap_O[]=
{//        01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40
    maprow(MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP),
    maprow(MP,RK,ER,RK,RK,ER,ER,ES,ES,RK,ER,ER,RK,ER,ER,RK,ER,RK,ER,ER,SP,RK,ER,ER,RK,ER,RK,RK,ER,ER,RK,ER,RK,RK,ER,ER,ER,RK,ER,MP),
    maprow(MP,ER,BR,ER,RK,RK,ER,ER,ER,ER,ER,ER,RK,ER,ER,RK,ER,ER,ER,ES,ER,ER,ER,ER,BR,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,RK,ER,RK,ER,MP),
    maprow(MP,RK,RK,BR,ER,RK,ER,ER,ER,ES,ES,ER,ER,ES,RK,ER,ER,RK,ER,RK,ER,ER,RK,BR,RK,ER,ER,ER,ER,ES,ER,ER,ES,ER,ER,RK,ER,RK,BM,MP),
    maprow(MP,ER,ER,ER,BR,RK,ER,ER,RK,ER,RK,BM,ER,ER,ER,ER,ER,RK,ER,RK,ER,ER,BR,RK,ER,ES,ES,ER,ER,ER,ER,ER,RK,RK,RK,RK,ER,ER,RK,MP),
    maprow(MP,ER,ES,RK,ER,BR,RK,RK,ER,ER,ER,ES,RK,ER,ER,RK,ER,RK,ER,ER,ER,BR,RK,ER,ES,ES,ER,ER,ER,ER,ER,ER,RK,RK,RK,RK,ER,ER,RK,MP),
    maprow(MP,ER,RK,RK,ER,ER,BR,RK,ER,ER,ES,ER,RK,ES,ER,ER,ES,ER,RK,RK,BR,ER,ER,ER,RK,ES,RK,RK,ER,ES,ER,ER,ER,RK,RK,RK,ER,ER,RK,MP),
    maprow(MP,ER,ER,ER,RK,RK,ER,BR,ER,BM,RK,ER,ER,RK,ER,ER,ES,ER,RK,BR,ER,ER,ER,ER,ER,RK,ER,RK,RK,ER,ER,ES,ER,RK,ER,RK,RK,ER,ER,MP),
    maprow(MP,ER,ER,RK,ER,RK,ER,ER,BR,ER,RK,RK,ER,ER,ER,ER,RK,ER,BR,ER,ER,ER,ES,ER,ER,RK,RK,RK,RK,ER,ER,RK,ER,ER,ER,ER,RK,ER,ER,MP),
    maprow(MP,ER,ER,ER,ES,ER,RK,RK,ER,BR,ER,ER,ER,ER,RK,ER,RK,BR,BM,RK,RK,ER,ES,ER,ER,ER,RK,RK,RK,ER,ER,RK,ER,RK,ER,ER,ER,ES,RK,MP),
    maprow(MP,RK,ER,RK,ER,ER,ER,RK,ER,RK,BR,ER,ER,ER,ER,ER,BR,ES,ER,ER,RK,ER,RK,ER,RK,ER,ER,ER,ER,ES,ER,RK,ES,ER,ER,RK,ER,RK,RK,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,RK,ER,RK,RK,BR,ER,ER,ER,BR,ER,RK,ER,ER,RK,RK,ER,ER,RK,ER,ER,ER,ER,RK,ER,ER,ER,RK,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,RK,RK,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,RK,ER,ER,ER,RK,ER,ER,RK,ER,RK,ES,ER,ER,RK,ER,RK,RK,ER,ER,ER,ES,RK,ER,ER,MP),
    maprow(MP,ER,RK,RK,ER,ER,ER,ER,ER,ER,RK,ER,MB,MB,MB,ER,ER,RK,ER,ER,ER,ER,RK,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,RK,ER,RK,RK,ER,ER,MP),
    maprow(MP,ER,ER,RK,ER,ER,ER,ES,ES,ER,ER,ER,RK,ER,ER,RK,ER,RK,ER,ER,RK,ER,RK,ES,ER,ER,ER,ES,ER,ER,ER,ER,ER,RK,ER,RK,RK,ER,ER,MP),
    maprow(MP,ER,ER,RK,ER,ER,RK,ER,RK,ER,ES,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,RK,ER,ES,ES,ER,ER,ER,ER,ER,ER,ES,RK,RK,ER,ER,RK,ER,ES,MP),
    maprow(MP,RK,ER,ER,ES,RK,ER,ER,ER,ER,RK,ER,ER,RK,ER,RK,ER,ER,ER,ER,RK,ER,ES,ES,ER,ER,ER,ER,ER,ER,ER,RK,RK,RK,ER,ER,RK,ER,ER,MP),
    maprow(MP,RK,ER,ER,ER,RK,ER,ER,RK,ER,RK,ES,ER,ER,RK,ER,ES,ES,ER,ER,ER,ER,RK,RK,RK,ES,RK,ER,ER,RK,ER,RK,ER,RK,ES,ER,ER,ES,RK,MP),
    maprow(MP,ER,ES,RK,ER,ER,BM,ES,RK,ER,ER,ER,ER,RK,ER,RK,RK,ER,ER,ER,ER,ER,ER,RK,RK,RK,RK,ER,ER,RK,ER,RK,ER,RK,RK,ER,ER,RK,RK,MP),
    maprow(MP,RK,ER,RK,RK,ER,ER,RK,ER,RK,RK,ER,ER,ER,ES,ER,RK,ER,RK,ER,ES,ER,ER,ER,RK,RK,RK,ER,ER,RK,ER,ER,ER,ER,RK,ER,ER,ER,RK,MP),
    maprow(MP,ER,ER,ER,RK,RK,ER,RK,ER,RK,RK,ER,ER,ER,ES,ER,RK,ER,RK,ER,EX,ER,ER,ER,RK,RK,RK,ER,ER,RK,ER,RK,ER,ER,ER,ER,RK,ER,ER,MP),
    maprow(MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP)};

char cavemap_P[]=
{//        01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40
    maprow(MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP),
    maprow(MP,SP,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,RK,ER,ER,RK,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,RK,ER,RK,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,RK,ER,RK,ER,EX),
    maprow(MP,ER,RK,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,RK,ER,ER,ER,ER,RK,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,RK,RK,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,RK,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,RK,ER,RK,ER,ER,RK,MP),
    maprow(MP,ER,ER,RK,ER,ER,ER,RK,ER,ER,ER,ER,RK,ER,ER,RK,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,RK,ER,ER,RK,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,RK,RK,RK,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,RK,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,RK,RK,ER,ER,RK,ER,ER,ER,ER,RK,ER,ER,MP),
    maprow(MP,ER,ER,RK,ER,ER,ER,MB,MB,MB,MB,MB,MB,ER,ER,MB,MB,MB,MB,MB,MB,ER,ER,ER,ER,ER,RK,ER,RK,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,RK,BR,ER,ER,ER,ER,BR,ER,ER,BR,ER,ER,RK,RK,BR,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,RK,ER,ER,ER,ER,RK,MP),
    maprow(MP,RK,ER,RK,ER,ER,ER,BR,ER,ER,RK,ER,BR,ER,ER,BR,ER,ER,ER,ER,BR,ER,ER,ER,RK,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,RK,ER,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,BR,ER,ER,RK,ER,BR,ER,ER,BR,ER,ER,ER,ER,BR,RK,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,MP),
    maprow(MP,ER,RK,RK,ER,ER,ER,BR,ER,ER,ER,ER,BR,RK,ER,BR,ER,ER,ER,ER,BR,ER,RK,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,MP),
    maprow(MP,ER,RK,RK,ER,ER,ER,BR,ER,ER,ER,ER,BR,RK,RK,BR,ER,RK,ER,ER,BR,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,RK,ER,RK,RK,ER,ER,MP),
    maprow(MP,ER,ER,RK,ER,ER,ER,BR,ER,ER,ER,ER,BR,ER,ER,BR,ER,ER,ER,ER,BR,ER,RK,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,RK,RK,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,RK,BR,BR,BR,BR,BR,BR,ER,ER,BR,BR,BR,BR,BR,BR,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,RK,ER,ER,MP),
    maprow(MP,RK,ER,ER,ER,RK,ER,ER,ER,ER,RK,ER,ER,RK,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,RK,ER,ER,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,RK,MP),
    maprow(MP,ER,ER,RK,ER,ER,RK,ER,ER,ES,ES,ER,ER,ER,ER,ER,ER,ES,ES,ER,ER,ER,ER,RK,ER,ES,ES,ER,ER,RK,ER,ER,ER,ES,ES,ER,ER,ER,RK,MP),
    maprow(MP,RK,ER,ER,ER,ER,ER,RK,ER,BM,ES,ER,ER,ER,ER,ER,ER,BM,ES,ER,ER,ER,ER,ER,ER,BM,ES,ER,ER,RK,ER,ER,ER,BM,ES,ER,ER,ER,RK,MP),
    maprow(MP,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,ER,RK,ER,ER,ER,ER,RK,ER,ER,MP),
    maprow(MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP,MP)};

#define cave_count  16
#define cave_width  40 //make sure this is EVEN
#define cave_height 22

//                                A   B   C   D   E   F   G   H   I   J   K   L   M   N   O   P
long cave_diamonds[cave_count]={ 12, 10, 24, 36,  4,  4, 15, 10, 75, 12,  6, 19, 50, 30, 15, 12};
long cave_time    [cave_count]={150,150,150,120,150,150,120,120,150,150,180,200,160,150,120,150};

char *cave_maps[cave_count]=
{
    cavemap_A,cavemap_B,cavemap_C,cavemap_D,cavemap_E,
    cavemap_F,cavemap_G,cavemap_H,cavemap_I,cavemap_J,
    cavemap_K,cavemap_L,cavemap_M,cavemap_N,cavemap_O,
    cavemap_P
};

//*****************************************************************************
#define BIN(_n) (char)( ((0x##_n##L>> 0)&  1)| \
                        ((0x##_n##L>> 3)&  2)| \
                        ((0x##_n##L>> 6)&  4)| \
                        ((0x##_n##L>> 9)&  8)| \
                        ((0x##_n##L>>12)& 16)| \
                        ((0x##_n##L>>15)& 32)| \
                        ((0x##_n##L>>18)& 64)| \
                        ((0x##_n##L>>21)&128))

char char_PI1[8]= //player idle
{
    BIN(00111100),
    BIN(01011010),
    BIN(00111100),
    BIN(00011000),
    BIN(01111110),
    BIN(00011000),
    BIN(00111100),
    BIN(01100110),
};
char char_PI2[8]= //player idle
{
    BIN(00111100),
    BIN(01011010),
    BIN(00111100),
    BIN(00011000),
    BIN(01111110),
    BIN(00011000),
    BIN(01111100),
    BIN(00000110),
};

char char_PL1[8]= //player left
{
    BIN(00111000),
    BIN(01011100),
    BIN(00111000),
    BIN(00011000),
    BIN(01111000),
    BIN(00011000),
    BIN(00110110),
    BIN(01100010),
};
char char_PL2[8]= //player left
{
    BIN(00111000),
    BIN(01011100),
    BIN(00111000),
    BIN(00011000),
    BIN(01111000),
    BIN(00011000),
    BIN(00011000),
    BIN(00111000),
};

char char_PR1[8]= //player right
{
    BIN(00011100),
    BIN(00111010),
    BIN(00011100),
    BIN(00011000),
    BIN(00011110),
    BIN(00011000),
    BIN(01101100),
    BIN(01000110),
};
char char_PR2[8]= //player right
{
    BIN(00011100),
    BIN(00111010),
    BIN(00011100),
    BIN(00011000),
    BIN(00011110),
    BIN(00011000),
    BIN(00011000),
    BIN(00011100),
};

char char_PU1[8]= //player up
{
    BIN(00111100),
    BIN(01111110),
    BIN(00111100),
    BIN(00011000),
    BIN(01111110),
    BIN(00011000),
    BIN(01111100),
    BIN(00000110),
};
char char_PU2[8]= //player up
{
    BIN(00111100),
    BIN(01111110),
    BIN(00111100),
    BIN(00011000),
    BIN(01111110),
    BIN(00011000),
    BIN(00111110),
    BIN(01100000),
};

char char_PD1[8]= //player down
{
    BIN(00111100),
    BIN(01011010),
    BIN(00111100),
    BIN(00011000),
    BIN(01111110),
    BIN(00011000),
    BIN(00111110),
    BIN(01100000),
};
char char_PD2[8]= //player down
{
    BIN(00111100),
    BIN(01011010),
    BIN(00111100),
    BIN(00011000),
    BIN(01111110),
    BIN(00011000),
    BIN(01111100),
    BIN(00000110),
};

char char_ES1[8]= //empty space frame1
{
    BIN(00000000),
    BIN(00000000),
    BIN(00000000),
    BIN(00000000),
    BIN(00000000),
    BIN(00000000),
    BIN(00000000),
    BIN(00000000),
};
char char_ES2[8]= //empty space frame2
{
    BIN(10101010),
    BIN(01010101),
    BIN(10101010),
    BIN(01010101),
    BIN(10101010),
    BIN(01010101),
    BIN(10101010),
    BIN(01010101),
};

char char_BS[8]= //blocked space
{
    BIN(00000000),
    BIN(00000000),
    BIN(00000000),
    BIN(00000000),
    BIN(00000000),
    BIN(00000000),
    BIN(00000000),
    BIN(00000000),
};

char char_SP1[8]= //starting point frame1
{
    BIN(11111111),
    BIN(10000001),
    BIN(10000001),
    BIN(10000001),
    BIN(10000001),
    BIN(10000001),
    BIN(10000001),
    BIN(11111111),
};
char char_SP2[8]= //starting point frame2
{
    BIN(11111111),
    BIN(11111111),
    BIN(11111111),
    BIN(11111111),
    BIN(11111111),
    BIN(11111111),
    BIN(11111111),
    BIN(11111111),
};

char char_EX1[8]= //exit frame1
{
    BIN(11111111),
    BIN(11000011),
    BIN(10100101),
    BIN(10011001),
    BIN(10011001),
    BIN(10100101),
    BIN(11000011),
    BIN(11111111),
};
char char_EX2[8]= //exit frame2
{
    BIN(11111111),
    BIN(10111101),
    BIN(11011011),
    BIN(11100111),
    BIN(11100111),
    BIN(11011011),
    BIN(10111101),
    BIN(11111111),
};

char char_ER[8]= //earth (soil)
{
#if 0
    BIN(01101100),
    BIN(00010110),
    BIN(10100001),
    BIN(01010110),
    BIN(00101011),
    BIN(10100101),
    BIN(01010010),
    BIN(11001011),
#endif
    BIN(00010000),
    BIN(00000010),
    BIN(10000000),
    BIN(00001000),
    BIN(01000000),
    BIN(00000001),
    BIN(01000000),
    BIN(00000010),
};

char char_RK[8]= //rock
{
    BIN(00000000),
    BIN(00111000),
    BIN(01110100),
    BIN(11111010),
    BIN(11111110),
    BIN(11111110),
    BIN(01111110),
    BIN(00111100),
};

char char_BR[8]= //brick
{
    BIN(11111110),
    BIN(11111110),
    BIN(11111110),
    BIN(00000000),
    BIN(11110111),
    BIN(11110111),
    BIN(11110111),
    BIN(00000000),
};

char char_DI1[8]= //diamond frame1
{
    BIN(00011000),
    BIN(00110100),
    BIN(01111110),
    BIN(11010101),
    BIN(10000001),
    BIN(01010110),
    BIN(00111100),
    BIN(00011000),
};
char char_DI2[8]= //diamond frame2
{
    BIN(00011000),
    BIN(00111100),
    BIN(01010110),
    BIN(10000001),
    BIN(11010101),
    BIN(01111110),
    BIN(00110100),
    BIN(00011000),
};
char char_DI3[8]= //diamond frame3
{
    BIN(00011000),
    BIN(00101100),
    BIN(01000010),
    BIN(10101011),
    BIN(11111111),
    BIN(01101010),
    BIN(00100100),
    BIN(00011000),
};
char char_DI4[8]= //diamond frame4
{
    BIN(00011000),
    BIN(00100100),
    BIN(01101010),
    BIN(11111111),
    BIN(10101011),
    BIN(01000010),
    BIN(00101100),
    BIN(00011000),
};

char char_BF1[8]= //butterfly monster frame1
{
    BIN(10000001),
    BIN(11000011),
    BIN(11100111),
    BIN(11111111),
    BIN(11111111),
    BIN(11100111),
    BIN(11000011),
    BIN(10000001),
};
char char_BF2[8]= //butterfly monster frame2
{
    BIN(01000010),
    BIN(01000010),
    BIN(01100110),
    BIN(01111110),
    BIN(01111110),
    BIN(01100110),
    BIN(01000010),
    BIN(01000010),
};
char char_BF3[8]= //butterfly monster frame3
{
    BIN(00100100),
    BIN(00100100),
    BIN(00100100),
    BIN(00111100),
    BIN(00111100),
    BIN(00100100),
    BIN(00100100),
    BIN(00100100),
};

char char_GS1[8]= //growing slime frame1
{
    BIN(01100110),
    BIN(11111111),
    BIN(11101111),
    BIN(11001110),
    BIN(01111100),
    BIN(11110110),
    BIN(11111111),
    BIN(00110111),
};
char char_GS2[8]= //growing slime frame2
{
    BIN(11110110),
    BIN(11111111),
    BIN(11101111),
    BIN(11011111),
    BIN(01111110),
    BIN(11110111),
    BIN(11111111),
    BIN(01110111),
};

char char_BM1[8]= //box monster frame1
{
    BIN(11111110),
    BIN(10000010),
    BIN(10111010),
    BIN(10101010),
    BIN(10111010),
    BIN(10000010),
    BIN(11111110),
    BIN(00000000),
};
char char_BM2[8]= //box monster frame2
{
    BIN(00000000),
    BIN(01111100),
    BIN(01000100),
    BIN(01010100),
    BIN(01000100),
    BIN(01111100),
    BIN(00000000),
    BIN(00000000),
};

char char_MP[8]= //metal plate
{
    BIN(11111111),
    BIN(10011001),
    BIN(10011001),
    BIN(11111111),
    BIN(11111111),
    BIN(10011001),
    BIN(10011001),
    BIN(11111111),
};

char char_CR1[8]= //cursor frame1
{
    BIN(00000000),
    BIN(00001000),
    BIN(00001100),
    BIN(00111110),
    BIN(00001100),
    BIN(00001000),
    BIN(00000000),
    BIN(00000000),
};
char char_CR2[8]= //cursor frame2
{
    BIN(00000000),
    BIN(00010000),
    BIN(00011000),
    BIN(01111100),
    BIN(00011000),
    BIN(00010000),
    BIN(00000000),
    BIN(00000000),
};
char char_CR3[8]= //cursor frame3
{
    BIN(00000000),
    BIN(00100000),
    BIN(00110000),
    BIN(11111000),
    BIN(00110000),
    BIN(00100000),
    BIN(00000000),
    BIN(00000000),
};

char char_EP1[8]= //
{
    BIN(00000000),
    BIN(00000000),
    BIN(00011000),
    BIN(00111100),
    BIN(00011100),
    BIN(00001000),
    BIN(00000000),
    BIN(00000000),
};
char char_EP2[8]= //
{
    BIN(00011000),
    BIN(00111110),
    BIN(01111111),
    BIN(11101011),
    BIN(11110111),
    BIN(01101110),
    BIN(01111110),
    BIN(00111100),
};
char char_EP3[8]= //
{
    BIN(10010011),
    BIN(01000001),
    BIN(00000100),
    BIN(11011000),
    BIN(00010001),
    BIN(00100100),
    BIN(01000010),
    BIN(11000001),
};
char char_EP4[8]= //
{
    BIN(00000000),
    BIN(00000010),
    BIN(00000000),
    BIN(01000100),
    BIN(00000000),
    BIN(00001000),
    BIN(01000000),
    BIN(00000000),
};

char char_MB1[8]= //magic brick frame1
{
    BIN(11111110),
    BIN(11111110),
    BIN(11111110),
    BIN(00000011),
    BIN(11110111),
    BIN(11110111),
    BIN(11110111),
    BIN(11000000),
};
char char_MB2[8]= //magic brick frame2
{
    BIN(11111110),
    BIN(11111110),
    BIN(11111110),
    BIN(00001100),
    BIN(11110111),
    BIN(11110111),
    BIN(11110111),
    BIN(00110000),
};
char char_MB3[8]= //magic brick frame3
{
    BIN(11111110),
    BIN(11111110),
    BIN(11111110),
    BIN(00110000),
    BIN(11110111),
    BIN(11110111),
    BIN(11110111),
    BIN(00001100),
};
char char_MB4[8]= //magic brick frame4
{
    BIN(11111110),
    BIN(11111110),
    BIN(11111110),
    BIN(11000000),
    BIN(11110111),
    BIN(11110111),
    BIN(11110111),
    BIN(00000011),
};

//*****************************************************************************
char char_font0[8]= //
{
    BIN(01111100),
    BIN(10000010),
    BIN(10000010),
    BIN(10000010),
    BIN(10000010),
    BIN(10000010),
    BIN(01111100),
    BIN(00000000),
};

char char_font1[8]= //
{
    BIN(00010000),
    BIN(00110000),
    BIN(00010000),
    BIN(00010000),
    BIN(00010000),
    BIN(00010000),
    BIN(01111100),
    BIN(00000000),
};

char char_font2[8]= //
{
    BIN(01111100),
    BIN(10000010),
    BIN(00000010),
    BIN(00011100),
    BIN(00100000),
    BIN(01000000),
    BIN(11111110),
    BIN(00000000),
};

char char_font3[8]= //
{
    BIN(01111100),
    BIN(10000010),
    BIN(00000010),
    BIN(00011100),
    BIN(00000010),
    BIN(10000010),
    BIN(01111100),
    BIN(00000000),
};

char char_font4[8]= //
{
    BIN(00001100),
    BIN(00010100),
    BIN(00100100),
    BIN(01000100),
    BIN(11111110),
    BIN(00000100),
    BIN(00000100),
    BIN(00000000),
};

char char_font5[8]= //
{
    BIN(11111100),
    BIN(10000000),
    BIN(10000000),
    BIN(11111100),
    BIN(00000010),
    BIN(10000010),
    BIN(01111100),
    BIN(00000000),
};

char char_font6[8]= //
{
    BIN(01111100),
    BIN(10000000),
    BIN(10000000),
    BIN(11111100),
    BIN(10000010),
    BIN(10000010),
    BIN(01111100),
    BIN(00000000),
};

char char_font7[8]= //
{
    BIN(11111110),
    BIN(00000010),
    BIN(00000100),
    BIN(00001000),
    BIN(00010000),
    BIN(00010000),
    BIN(00010000),
    BIN(00000000),
};

char char_font8[8]= //
{
    BIN(01111100),
    BIN(10000010),
    BIN(10000010),
    BIN(01111100),
    BIN(10000010),
    BIN(10000010),
    BIN(01111100),
    BIN(00000000),
};

char char_font9[8]= //
{
    BIN(01111100),
    BIN(10000010),
    BIN(10000010),
    BIN(01111110),
    BIN(00000010),
    BIN(00000010),
    BIN(01111100),
    BIN(00000000),
};

char char_fontA[8]= //
{
    BIN(00010000),
    BIN(00101000),
    BIN(01000100),
    BIN(10000010),
    BIN(11111110),
    BIN(10000010),
    BIN(10000010),
    BIN(00000000),
};

char char_fontB[8]= //
{
    BIN(11111100),
    BIN(10000010),
    BIN(10000010),
    BIN(11111100),
    BIN(10000010),
    BIN(10000010),
    BIN(11111100),
    BIN(00000000),
};

char char_fontC[8]= //
{
    BIN(00111100),
    BIN(01000010),
    BIN(10000000),
    BIN(10000000),
    BIN(10000000),
    BIN(01000010),
    BIN(00111100),
    BIN(00000000),
};

char char_fontD[8]= //
{
    BIN(11111000),
    BIN(10000100),
    BIN(10000010),
    BIN(10000010),
    BIN(10000010),
    BIN(10000100),
    BIN(11111000),
    BIN(00000000),
};

char char_fontE[8]= //
{
    BIN(11111110),
    BIN(10000000),
    BIN(10000000),
    BIN(11111000),
    BIN(10000000),
    BIN(10000000),
    BIN(11111110),
    BIN(00000000),
};

char char_fontF[8]= //
{
    BIN(11111110),
    BIN(10000000),
    BIN(10000000),
    BIN(11111000),
    BIN(10000000),
    BIN(10000000),
    BIN(10000000),
    BIN(00000000),
};

char char_fontG[8]= //
{
    BIN(00111100),
    BIN(01000010),
    BIN(10000000),
    BIN(10011110),
    BIN(10000010),
    BIN(01000010),
    BIN(00111100),
    BIN(00000000),
};

char char_fontH[8]= //
{
    BIN(10000010),
    BIN(10000010),
    BIN(10000010),
    BIN(11111110),
    BIN(10000010),
    BIN(10000010),
    BIN(10000010),
    BIN(00000000),
};

char char_fontI[8]= //
{
    BIN(01111100),
    BIN(00010000),
    BIN(00010000),
    BIN(00010000),
    BIN(00010000),
    BIN(00010000),
    BIN(01111100),
    BIN(00000000),
};

char char_fontJ[8]= //
{
    BIN(00111110),
    BIN(00000100),
    BIN(00000100),
    BIN(00000100),
    BIN(00000100),
    BIN(10000100),
    BIN(01111000),
    BIN(00000000),
};

char char_fontK[8]= //
{
    BIN(10000010),
    BIN(10000100),
    BIN(10001000),
    BIN(11110000),
    BIN(10001000),
    BIN(10000100),
    BIN(10000010),
    BIN(00000000),
};

char char_fontL[8]= //
{
    BIN(10000000),
    BIN(10000000),
    BIN(10000000),
    BIN(10000000),
    BIN(10000000),
    BIN(10000000),
    BIN(11111110),
    BIN(00000000),
};

char char_fontM[8]= //
{
    BIN(10000010),
    BIN(11000110),
    BIN(10101010),
    BIN(10010010),
    BIN(10000010),
    BIN(10000010),
    BIN(10000010),
    BIN(00000000),
};

char char_fontN[8]= //
{
    BIN(10000010),
    BIN(11000010),
    BIN(10100010),
    BIN(10010010),
    BIN(10001010),
    BIN(10000110),
    BIN(10000010),
    BIN(00000000),
};

char char_fontO[8]= //
{
    BIN(00111000),
    BIN(01000100),
    BIN(10000010),
    BIN(10000010),
    BIN(10000010),
    BIN(01000100),
    BIN(00111000),
    BIN(00000000),
};

char char_fontP[8]= //
{
    BIN(11111100),
    BIN(10000010),
    BIN(10000010),
    BIN(11111100),
    BIN(10000000),
    BIN(10000000),
    BIN(10000000),
    BIN(00000000),
};

char char_fontQ[8]= //
{
    BIN(00111000),
    BIN(01000100),
    BIN(10000010),
    BIN(10000010),
    BIN(10001010),
    BIN(01000100),
    BIN(00111010),
    BIN(00000000),
};

char char_fontR[8]= //
{
    BIN(11111100),
    BIN(10000010),
    BIN(10000010),
    BIN(11111100),
    BIN(10001000),
    BIN(10000100),
    BIN(10000010),
    BIN(00000000),
};

char char_fontS[8]= //
{
    BIN(01111100),
    BIN(10000010),
    BIN(10000000),
    BIN(01111100),
    BIN(00000010),
    BIN(10000010),
    BIN(01111100),
    BIN(00000000),
};

char char_fontT[8]= //
{
    BIN(11111110),
    BIN(00010000),
    BIN(00010000),
    BIN(00010000),
    BIN(00010000),
    BIN(00010000),
    BIN(00010000),
    BIN(00000000),
};

char char_fontU[8]= //
{
    BIN(10000010),
    BIN(10000010),
    BIN(10000010),
    BIN(10000010),
    BIN(10000010),
    BIN(10000010),
    BIN(01111100),
    BIN(00000000),
};

char char_fontV[8]= //
{
    BIN(10000010),
    BIN(10000010),
    BIN(10000010),
    BIN(10000010),
    BIN(01000100),
    BIN(00101000),
    BIN(00010000),
    BIN(00000000),
};

char char_fontW[8]= //
{
    BIN(10000010),
    BIN(10000010),
    BIN(10000010),
    BIN(10010010),
    BIN(10101010),
    BIN(11000110),
    BIN(10000010),
    BIN(00000000),
};

char char_fontX[8]= //
{
    BIN(10000010),
    BIN(01000100),
    BIN(00101000),
    BIN(00010000),
    BIN(00101000),
    BIN(01000100),
    BIN(10000010),
    BIN(00000000),
};

char char_fontY[8]= //
{
    BIN(10000010),
    BIN(10000010),
    BIN(01000100),
    BIN(00101000),
    BIN(00010000),
    BIN(00010000),
    BIN(00010000),
    BIN(00000000),
};

char char_fontZ[8]= //
{
    BIN(11111110),
    BIN(00000100),
    BIN(00001000),
    BIN(00010000),
    BIN(00100000),
    BIN(01000000),
    BIN(11111110),
    BIN(00000000),
};

char char_fontPERIOD[8]= //
{
    BIN(00000000),
    BIN(00000000),
    BIN(00000000),
    BIN(00000000),
    BIN(00000000),
    BIN(00000000),
    BIN(01000000),
    BIN(00000000),
};

char char_fontCOMMA[8]= //
{
    BIN(00000000),
    BIN(00000000),
    BIN(00000000),
    BIN(00000000),
    BIN(00000000),
    BIN(00100000),
    BIN(01000000),
    BIN(00000000),
};

char char_fontHYPHEN[8]= //
{
    BIN(00000000),
    BIN(00000000),
    BIN(00000000),
    BIN(01111100),
    BIN(00000000),
    BIN(00000000),
    BIN(00000000),
    BIN(00000000),
};

char char_fontCOLON[8]= //
{
    BIN(00000000),
    BIN(00000000),
    BIN(01000000),
    BIN(00000000),
    BIN(00000000),
    BIN(01000000),
    BIN(00000000),
    BIN(00000000),
};

char char_fontEXCL[8]= //
{
    BIN(01000000),
    BIN(01000000),
    BIN(01000000),
    BIN(01000000),
    BIN(01000000),
    BIN(00000000),
    BIN(01000000),
    BIN(00000000),
};

char char_fontFWSLASH[8]= //
{
    BIN(00000010),
    BIN(00000100),
    BIN(00001000),
    BIN(00010000),
    BIN(00100000),
    BIN(01000000),
    BIN(10000000),
    BIN(00000000),
};

char char_fontAPOST[8]= //
{
    BIN(00010000),
    BIN(00100000),
    BIN(00000000),
    BIN(00000000),
    BIN(00000000),
    BIN(00000000),
    BIN(00000000),
    BIN(00000000),
};

char char_fontSPACE[8]= //
{
    BIN(00000000),
    BIN(00000000),
    BIN(00000000),
    BIN(00000000),
    BIN(00000000),
    BIN(00000000),
    BIN(00000000),
    BIN(00000000),
};

//*****************************************************************************
//char property flags
#define cf_nomove    (1<< 0) //cant move/push this
#define cf_death     (1<< 1) //touching this kills you
#define cf_push      (1<< 2) //able to push this into empty space
#define cf_exit      (1<< 3) //exit door
#define cf_nodestory (1<< 4) //dont destroy in explosions
#define cf_diamond   (1<< 5) //mmm diamonds
#define cf_falls     (1<< 6) //this can fall
#define cf_slime     (1<< 7) //this is slime, monsters explode on contact
#define cf_nosupport (1<< 8) //does not support rocks/diamonds, they fall off
#define cf_convrock  (1<< 9) //converts falling rocks into diamonds
#define cf_ai        (1<<10) //this requires AI processing, eg: slime, monster
#define cf_standgrab (1<<11) //can stand and grab this
#define cf_exploding (1<<12) //part of an explosion
#define cf_slimegrow (1<<13) //slime can grow into this

struct
{
    long flags;
    char animframes;
    char *bitmap[4];
} char_info[CHAR_COUNT]=
{
    {cf_slimegrow                                 ,1,{char_ES1,char_ES2,0,0}}, //empty space
    {cf_nomove|cf_nodestory                       ,1,{char_MP,0,0,0}}, //metal plate
    {0                                            ,2,{char_SP1,char_SP2,0,0}}, //starting point
    {cf_exit|cf_nosupport                         ,2,{char_EX1,char_EX2,0,0}}, //exit
    {cf_standgrab|cf_slimegrow                    ,1,{char_ER,0,0,0}}, //earth (soil)
    {cf_push|cf_falls|cf_nosupport                ,1,{char_RK,0,0,0}}, //rock
    {cf_nomove|cf_nosupport                       ,1,{char_BR,0,0,0}}, //brick
    {cf_diamond|cf_falls|cf_nosupport|cf_standgrab,4,{char_DI1,char_DI2,char_DI3,char_DI4}}, //diamond
    {cf_death|cf_ai                               ,4,{char_BF1,char_BF2,char_BF3,char_BF2}}, //butterfly monster
    {cf_nomove|cf_ai|cf_slime                     ,4,{char_GS1,char_GS1,char_GS1,char_GS2}}, //growing slime
    {cf_death|cf_ai                               ,2,{char_BM1,char_BM2,0,0}}, //box monster
    {cf_nomove|cf_nosupport|cf_convrock           ,4,{char_MB1,char_MB2,char_MB3,char_MB4}}, //magic brick
    {0,1,{char_fontSPACE,0,0,0}}, //_12 unused
    {0,1,{char_fontSPACE,0,0,0}}, //_13 unused
    {0,1,{char_fontSPACE,0,0,0}}, //_14 unused
    {0,1,{char_fontSPACE,0,0,0}}, //_15 unused

    {cf_nomove                                    ,1,{char_BS,0,0,0}}, //blocked space
    {0                                            ,4,{char_PI1,char_PI1,char_PI1,char_PI2}}, //player idle
    {0                                            ,2,{char_PL1,char_PL2,0,0}}, //player left
    {0                                            ,2,{char_PR1,char_PR2,0,0}}, //player right
    {0                                            ,2,{char_PU1,char_PU2,0,0}}, //player up
    {0                                            ,2,{char_PD1,char_PD2,0,0}}, //player down
    {0                                            ,4,{char_CR1,char_CR2,char_CR3,char_CR2}}, //cursor
    {cf_death|cf_exploding                        ,1,{char_EP1,0,0,0}}, //explode1
    {cf_death|cf_exploding                        ,1,{char_EP2,0,0,0}}, //explode2
    {cf_death|cf_exploding                        ,1,{char_EP3,0,0,0}}, //explode3
    {cf_death|cf_exploding                        ,1,{char_EP4,0,0,0}}, //explode4

    {0,1,{char_fontSPACE,0,0,0}}, //_27 unused
    {0,1,{char_fontSPACE,0,0,0}}, //_28 unused
    {0,1,{char_fontSPACE,0,0,0}}, //_29 unused
    {0,1,{char_fontSPACE,0,0,0}}, //_30 unused
    {0,1,{char_fontSPACE,0,0,0}}, //_31 unused

    {0,1,{char_fontSPACE,0,0,0}}, //_32
    {0,1,{char_fontEXCL,0,0,0}}, //_33
    {0,1,{char_fontSPACE,0,0,0}}, //_34
    {0,1,{char_fontSPACE,0,0,0}}, //_35
    {0,1,{char_fontSPACE,0,0,0}}, //_36
    {0,1,{char_fontSPACE,0,0,0}}, //_37
    {0,1,{char_fontSPACE,0,0,0}}, //_38
    {0,1,{char_fontAPOST,0,0,0}}, //_39
    {0,1,{char_fontSPACE,0,0,0}}, //_40
    {0,1,{char_fontSPACE,0,0,0}}, //_41
    {0,1,{char_fontSPACE,0,0,0}}, //_42
    {0,1,{char_fontSPACE,0,0,0}}, //_43
    {0,1,{char_fontCOMMA,0,0,0}}, //_44
    {0,1,{char_fontHYPHEN,0,0,0}}, //_45
    {0,1,{char_fontPERIOD,0,0,0}}, //_46
    {0,1,{char_fontFWSLASH,0,0,0}}, //_47
    {0,1,{char_font0,0,0,0}}, //_48
    {0,1,{char_font1,0,0,0}}, //_49
    {0,1,{char_font2,0,0,0}}, //_50
    {0,1,{char_font3,0,0,0}}, //_51
    {0,1,{char_font4,0,0,0}}, //_52
    {0,1,{char_font5,0,0,0}}, //_53
    {0,1,{char_font6,0,0,0}}, //_54
    {0,1,{char_font7,0,0,0}}, //_55
    {0,1,{char_font8,0,0,0}}, //_56
    {0,1,{char_font9,0,0,0}}, //_57
    {0,1,{char_fontCOLON,0,0,0}}, //_58
    {0,1,{char_fontSPACE,0,0,0}}, //_59
    {0,1,{char_fontSPACE,0,0,0}}, //_60
    {0,1,{char_fontSPACE,0,0,0}}, //_61
    {0,1,{char_fontSPACE,0,0,0}}, //_62
    {0,1,{char_fontSPACE,0,0,0}}, //_63
    {0,1,{char_fontSPACE,0,0,0}}, //_64
    {0,1,{char_fontA,0,0,0}}, //_65
    {0,1,{char_fontB,0,0,0}}, //_66
    {0,1,{char_fontC,0,0,0}}, //_67
    {0,1,{char_fontD,0,0,0}}, //_68
    {0,1,{char_fontE,0,0,0}}, //_69
    {0,1,{char_fontF,0,0,0}}, //_70
    {0,1,{char_fontG,0,0,0}}, //_71
    {0,1,{char_fontH,0,0,0}}, //_72
    {0,1,{char_fontI,0,0,0}}, //_73
    {0,1,{char_fontJ,0,0,0}}, //_74
    {0,1,{char_fontK,0,0,0}}, //_75
    {0,1,{char_fontL,0,0,0}}, //_76
    {0,1,{char_fontM,0,0,0}}, //_77
    {0,1,{char_fontN,0,0,0}}, //_78
    {0,1,{char_fontO,0,0,0}}, //_79
    {0,1,{char_fontP,0,0,0}}, //_80
    {0,1,{char_fontQ,0,0,0}}, //_81
    {0,1,{char_fontR,0,0,0}}, //_82
    {0,1,{char_fontS,0,0,0}}, //_83
    {0,1,{char_fontT,0,0,0}}, //_84
    {0,1,{char_fontU,0,0,0}}, //_85
    {0,1,{char_fontV,0,0,0}}, //_86
    {0,1,{char_fontW,0,0,0}}, //_87
    {0,1,{char_fontX,0,0,0}}, //_88
    {0,1,{char_fontY,0,0,0}}, //_89
    {0,1,{char_fontZ,0,0,0}}, //_90
};

//char data flags
#define cdf_isfalling   (1<<0)
#define cdf_moveup      (0<<1)
#define cdf_movedown    (1<<1)
#define cdf_moveleft    (2<<1)
#define cdf_moveright   (3<<1)
#define cdf_movemask    (cdf_moveup|cdf_movedown|cdf_moveleft|cdf_moveright)
#define cdf_slimeedge   (1<<3)
#define cdf_aiupdated   (1<<4)
#define cdf_turnedtoES  (1<<5)
#define cdf_makediamond (1<<6) //flags an explosion to convert to diamond when done
